-- MySQL dump 8.22
--
-- Host: localhost    Database: genomics_test
---------------------------------------------------------
-- Server version	4.0.17-standard-log

--
-- Table structure for table 'AASeq'
--

CREATE TABLE AASeq (
  locusId int(10) unsigned default NULL,
  version int(2) unsigned NOT NULL default '1',
  sequence longblob,
  UNIQUE KEY Orf_Version (locusId,version)
) TYPE=MyISAM;

--
-- Table structure for table 'ArrayData'
--

CREATE TABLE ArrayData (
  spotId int(10) unsigned NOT NULL default '0',
  subExpId int(10) unsigned NOT NULL default '0',
  log2Ratio float NOT NULL default '0',
  log2RatioNormalized float default NULL,
  control float default NULL,
  treatment float default NULL,
  dyeCorr float default NULL,
  leftPix int(10) default NULL,
  rightPix int(10) default NULL,
  topPix int(10) default NULL,
  bottomPix int(10) default NULL,
  KEY spotId (spotId),
  KEY subExpId (subExpId)
) TYPE=MyISAM;

--
-- Table structure for table 'ArrayDataAttrs'
--

CREATE TABLE ArrayDataAttrs (
  spotId int(10) unsigned NOT NULL default '0',
  subExpId int(10) unsigned NOT NULL default '0',
  attr varchar(20) default NULL,
  value varchar(20) default NULL,
  KEY spotId (spotId),
  KEY subExpId (subExpId)
) TYPE=MyISAM;

--
-- Table structure for table 'AutoAnno'
--

CREATE TABLE AutoAnno (
  db varchar(20) default NULL,
  locusId int(10) unsigned default NULL,
  version int(2) unsigned default '1',
  subject longtext,
  identity float unsigned NOT NULL default '0',
  alignLength int(10) unsigned NOT NULL default '0',
  mismatch int(10) unsigned NOT NULL default '0',
  gap int(10) unsigned NOT NULL default '0',
  qBegin int(10) unsigned NOT NULL default '0',
  qEnd int(10) unsigned NOT NULL default '0',
  sBegin int(10) unsigned NOT NULL default '0',
  sEnd int(10) unsigned NOT NULL default '0',
  evalue float NOT NULL default '0',
  score float NOT NULL default '0',
  UNIQUE KEY combined (locusId,subject(200),identity,qBegin,score),
  KEY db (db)
) TYPE=MyISAM;

--
-- Table structure for table 'BLASTp'
--

CREATE TABLE BLASTp (
  locusId int(10) unsigned default NULL,
  version int(2) unsigned default '1',
  subject int(10) unsigned default NULL,
  sVersion int(2) unsigned default '1',
  identity decimal(5,2) unsigned NOT NULL default '0.00',
  alignLength int(10) unsigned NOT NULL default '0',
  mismatch int(10) unsigned NOT NULL default '0',
  gap int(10) unsigned NOT NULL default '0',
  qBegin int(10) unsigned NOT NULL default '0',
  qEnd int(10) unsigned NOT NULL default '0',
  sBegin int(10) unsigned NOT NULL default '0',
  sEnd int(10) unsigned NOT NULL default '0',
  evalue float unsigned NOT NULL default '0',
  score decimal(10,2) unsigned NOT NULL default '0.00',
  KEY orfId (locusId),
  KEY subject (subject)
) TYPE=MyISAM;

--
-- Table structure for table 'COG'
--

CREATE TABLE COG (
  cogId int(10) unsigned NOT NULL auto_increment,
  locusId int(10) unsigned default NULL,
  version int(2) unsigned NOT NULL default '1',
  cogInfoId int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (cogId),
  UNIQUE KEY combined (locusId,cogInfoId),
  UNIQUE KEY orfId (locusId,version),
  KEY cogInfoId (cogInfoId)
) TYPE=MyISAM;

--
-- Table structure for table 'COGFun'
--

CREATE TABLE COGFun (
  funCode char(1) default NULL,
  description varchar(70) default NULL,
  funGroup varchar(50) default NULL
) TYPE=MyISAM;

--
-- Table structure for table 'COGInfo'
--

CREATE TABLE COGInfo (
  cogInfoId int(10) unsigned NOT NULL default '0',
  funCode varchar(5) NOT NULL default '',
  description varchar(100) default NULL,
  PRIMARY KEY  (cogInfoId)
) TYPE=MyISAM;

--
-- Table structure for table 'ChipInfo'
--

CREATE TABLE ChipInfo (
  chipId int(10) unsigned NOT NULL auto_increment,
  taxonomyId int(10) NOT NULL default '0',
  source varchar(255) default NULL,
  chipType varchar(255) default NULL,
  description varchar(255) default NULL,
  PRIMARY KEY  (chipId)
) TYPE=MyISAM;

--
-- Table structure for table 'DeletedLocus'
--

CREATE TABLE DeletedLocus (
  locusId int(10) unsigned default NULL,
  begin int(10) unsigned default NULL,
  end int(10) unsigned default NULL,
  strand char(1) default NULL,
  gi varchar(20) default NULL,
  name1 varchar(100) default NULL,
  name2 varchar(100) default NULL,
  cogInfoId int(10) unsigned default NULL,
  description text,
  scaffoldId int(10) unsigned default NULL,
  deleted date default NULL
) TYPE=MyISAM;

--
-- Table structure for table 'Description'
--

CREATE TABLE Description (
  descriptionId int(10) unsigned NOT NULL auto_increment,
  description text,
  source varchar(255) default NULL,
  created date default NULL,
  locusId int(10) unsigned default NULL,
  version int(2) unsigned NOT NULL default '1',
  PRIMARY KEY  (descriptionId),
  KEY OrfId_version (locusId,version)
) TYPE=MyISAM;

--
-- Table structure for table 'Dimer'
--

CREATE TABLE Dimer (
  dimerId int(10) unsigned NOT NULL auto_increment,
  scaffoldId int(10) unsigned NOT NULL default '0',
  motif varchar(255) NOT NULL default '',
  PRIMARY KEY  (dimerId)
) TYPE=MyISAM;

--
-- Table structure for table 'DimerPos'
--

CREATE TABLE DimerPos (
  dimerId int(10) unsigned NOT NULL default '0',
  posId int(10) unsigned NOT NULL default '0',
  tuId int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (dimerId)
) TYPE=MyISAM;

--
-- Table structure for table 'ECInfo'
--

CREATE TABLE ECInfo (
  ecNum varchar(20) NOT NULL default '',
  name varchar(255) NOT NULL default '',
  PRIMARY KEY  (ecNum)
) TYPE=MyISAM;

--
-- Table structure for table 'ExpInfo'
--

CREATE TABLE ExpInfo (
  expId int(10) unsigned NOT NULL auto_increment,
  chipId int(10) unsigned NOT NULL default '0',
  sampleTaxId int(10) NOT NULL default '0',
  condition varchar(255) NOT NULL default '',
  controlInfo text NOT NULL,
  treatmentInfo text NOT NULL,
  pubMedId varchar(255) default NULL,
  citation varchar(255) default NULL,
  contact varchar(255) NOT NULL default '',
  email varchar(255) NOT NULL default '',
  comment text NOT NULL,
  PRIMARY KEY  (expId)
) TYPE=MyISAM;

--
-- Table structure for table 'GoCount'
--

CREATE TABLE GoCount (
  goId int(10) unsigned NOT NULL default '0',
  goCount int(10) unsigned NOT NULL default '0',
  taxId int(10) unsigned NOT NULL default '0',
  UNIQUE KEY tax2go (taxId,goId),
  KEY taxId (taxId),
  KEY goId (goId)
) TYPE=MyISAM;

--
-- Table structure for table 'HitInfo'
--

CREATE TABLE HitInfo (
  hitId varchar(100) NOT NULL default '',
  synId varchar(100) NOT NULL default '',
  sLength int(10) unsigned NOT NULL default '0',
  description longtext NOT NULL,
  PRIMARY KEY  (hitId)
) TYPE=MyISAM;

--
-- Table structure for table 'IPRInfo'
--

CREATE TABLE IPRInfo (
  iprId varchar(9) NOT NULL default '',
  type varchar(16) default NULL,
  shortName varchar(50) default NULL,
  proteinCount int(5) default NULL,
  iprName varchar(255) default NULL,
  PRIMARY KEY  (iprId),
  KEY type (type)
) TYPE=MyISAM;

--
-- Table structure for table 'InterPro'
--

CREATE TABLE InterPro (
  locusId int(10) unsigned default NULL,
  checksum varchar(16) default NULL,
  length int(5) unsigned default NULL,
  domainDb varchar(30) default NULL,
  domainId varchar(30) default NULL,
  domainDesc varchar(255) default NULL,
  domainStart int(5) default NULL,
  domainEnd int(5) default NULL,
  evalue float default NULL,
  field varchar(10) default NULL,
  date varchar(50) default NULL,
  iprId varchar(9) default NULL,
  iprName varchar(255) default NULL,
  geneOntology longtext,
  KEY orfId (locusId),
  KEY iprId (iprId)
) TYPE=MyISAM;

--
-- Table structure for table 'JobRequest'
--

CREATE TABLE JobRequest (
  jobId varchar(255) NOT NULL default '',
  category enum('db','website','operon','regulon','microarray') default NULL,
  who varchar(50) default NULL,
  email varchar(255) default NULL,
  status enum('Submitted','In Progress','Completed') default NULL,
  jobDesc longtext,
  contact enum('khhuang','ejalm','mnprice','rpkoche','kkeller') default NULL,
  updated timestamp(10) NOT NULL,
  created timestamp(10) NOT NULL,
  log longtext,
  PRIMARY KEY  (jobId),
  KEY category (category),
  KEY who (who),
  KEY contact (contact)
) TYPE=MyISAM;

--
-- Table structure for table 'KEGGConf'
--

CREATE TABLE KEGGConf (
  mapId varchar(20) default NULL,
  object varchar(20) NOT NULL default '',
  type int(1) NOT NULL default '0',
  url varchar(255) NOT NULL default '',
  coord varchar(100) default NULL,
  UNIQUE KEY combo (mapId,object,coord),
  KEY mapId (mapId),
  KEY object (object)
) TYPE=MyISAM;

--
-- Table structure for table 'KEGGMap'
--

CREATE TABLE KEGGMap (
  mapId varchar(20) NOT NULL default '',
  title varchar(255) NOT NULL default '',
  PRIMARY KEY  (mapId)
) TYPE=MyISAM;

--
-- Table structure for table 'Locus'
--

CREATE TABLE Locus (
  locusId int(10) unsigned NOT NULL auto_increment,
  version int(2) unsigned NOT NULL default '1',
  priority tinyint(3) unsigned default NULL,
  created date default NULL,
  posId int(10) unsigned default NULL,
  evidence varchar(20) default NULL,
  scaffoldId int(10) unsigned default NULL,
  PRIMARY KEY  (locusId,version),
  KEY Indx_ORF_PosId (posId),
  KEY Indx_ORF_OrfId (locusId),
  KEY scaffoldId (scaffoldId)
) TYPE=MyISAM;

--
-- Table structure for table 'Locus2Ec'
--

CREATE TABLE Locus2Ec (
  locusId int(10) unsigned default NULL,
  version int(2) unsigned default '1',
  scaffoldId int(10) unsigned NOT NULL default '0',
  ecNum varchar(20) NOT NULL default '',
  UNIQUE KEY combined (locusId,ecNum),
  KEY scaffoldId (scaffoldId),
  KEY ecNum (ecNum)
) TYPE=MyISAM;

--
-- Table structure for table 'Locus2Go'
--

CREATE TABLE Locus2Go (
  locusId int(10) unsigned default NULL,
  goId int(10) unsigned NOT NULL default '0',
  evidence varchar(255) NOT NULL default '',
  UNIQUE KEY orf2go (locusId,goId),
  KEY orfId (locusId),
  KEY goId (goId)
) TYPE=MyISAM;

--
-- Table structure for table 'Locus2Ipr'
--

CREATE TABLE Locus2Ipr (
  locusId int(10) unsigned default NULL,
  iprId varchar(9) NOT NULL default '',
  UNIQUE KEY orf2ipr (locusId,iprId),
  KEY orfId (locusId),
  KEY iprId (iprId)
) TYPE=MyISAM;

--
-- Table structure for table 'Locus2Operon'
--

CREATE TABLE Locus2Operon (
  locusId int(10) unsigned default NULL,
  tuId int(10) unsigned NOT NULL default '0',
  KEY tuId (tuId),
  KEY orfId (locusId)
) TYPE=MyISAM;

--
-- Table structure for table 'LocusSeq'
--

CREATE TABLE LocusSeq (
  locusId int(10) unsigned default NULL,
  version int(2) unsigned NOT NULL default '1',
  sequence longblob,
  UNIQUE KEY orfId_version (locusId,version),
  KEY Index_Orf (locusId)
) TYPE=MyISAM;

--
-- Table structure for table 'ObjectDescr'
--

CREATE TABLE ObjectDescr (
  objectId int(2) unsigned NOT NULL default '0',
  description varchar(255) default NULL,
  PRIMARY KEY  (objectId)
) TYPE=MyISAM;

--
-- Table structure for table 'OldTax'
--

CREATE TABLE OldTax (
  oldId int(10) unsigned NOT NULL default '0',
  taxonomyId int(10) unsigned NOT NULL default '0',
  KEY taxonomyId (taxonomyId),
  KEY oldId (oldId)
) TYPE=MyISAM;

--
-- Table structure for table 'Operon'
--

CREATE TABLE Operon (
  tuId int(10) unsigned NOT NULL auto_increment,
  posId int(10) unsigned NOT NULL default '0',
  evidence varchar(255) NOT NULL default '',
  PRIMARY KEY  (tuId),
  UNIQUE KEY posId (posId)
) TYPE=MyISAM;

--
-- Table structure for table 'Ortholog'
--

CREATE TABLE Ortholog (
  locus1 int(10) unsigned NOT NULL default '0',
  locus2 int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (locus1),
  UNIQUE KEY locus2 (locus2),
  UNIQUE KEY combined (locus1,locus2)
) TYPE=MyISAM;

--
-- Table structure for table 'Position'
--

CREATE TABLE Position (
  posId int(10) unsigned NOT NULL auto_increment,
  strand char(1) default NULL,
  begin int(10) unsigned default NULL,
  end int(10) unsigned default NULL,
  scaffoldId int(10) unsigned default NULL,
  objectId int(2) unsigned default '1',
  PRIMARY KEY  (posId),
  KEY Indx_Position_scaffoldId (scaffoldId),
  KEY objectId (objectId),
  KEY start (begin)
) TYPE=MyISAM;

--
-- Table structure for table 'Promoter'
--

CREATE TABLE Promoter (
  promoterId int(10) unsigned NOT NULL auto_increment,
  posId int(10) unsigned NOT NULL default '0',
  tuId int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (promoterId)
) TYPE=MyISAM;

--
-- Table structure for table 'Regulon'
--

CREATE TABLE Regulon (
  regId int(10) unsigned NOT NULL default '0',
  locusId int(10) unsigned NOT NULL default '0',
  type varchar(10) NOT NULL default '',
  cutoff int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (regId,locusId,type,cutoff),
  KEY orfId (locusId),
  KEY regId (regId),
  KEY type (type),
  KEY cutoff (cutoff)
) TYPE=MyISAM;

--
-- Table structure for table 'RegulonLocus'
--

CREATE TABLE RegulonLocus (
  regId int(10) unsigned NOT NULL default '0',
  locusId int(10) unsigned NOT NULL default '0',
  type varchar(10) NOT NULL default '',
  cutoff int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (regId,locusId,type,cutoff),
  KEY orfId (locusId),
  KEY regId (regId),
  KEY type (type),
  KEY cutoff (cutoff)
) TYPE=MyISAM;

--
-- Table structure for table 'Scaffold'
--

CREATE TABLE Scaffold (
  scaffoldId int(10) unsigned NOT NULL auto_increment,
  chr_num int(10) unsigned default '1',
  isCircular int(1) default NULL,
  length int(10) unsigned default NULL,
  file varchar(32) default NULL,
  isGenomic int(1) default NULL,
  gi int(10) unsigned default NULL,
  taxonomyId int(10) default '0',
  comment varchar(255) default NULL,
  isActive int(1) default '1',
  isPartial int(1) default '0',
  PRIMARY KEY  (scaffoldId),
  KEY Indx_Scaffold_file (file),
  KEY comment (comment),
  KEY taxonomyId (taxonomyId)
) TYPE=MyISAM;

--
-- Table structure for table 'ScaffoldSeq'
--

CREATE TABLE ScaffoldSeq (
  scaffoldId int(10) unsigned NOT NULL default '0',
  sequence longblob,
  PRIMARY KEY  (scaffoldId),
  KEY Index_Scaffold (scaffoldId)
) TYPE=MyISAM;

--
-- Table structure for table 'SpotAttrs'
--

CREATE TABLE SpotAttrs (
  spotId int(10) unsigned NOT NULL default '0',
  attr varchar(20) default NULL,
  value varchar(20) default NULL
) TYPE=MyISAM;

--
-- Table structure for table 'SpotInfo'
--

CREATE TABLE SpotInfo (
  spotId int(10) unsigned NOT NULL auto_increment,
  chipId int(10) unsigned NOT NULL default '0',
  sourceId varchar(255) default NULL,
  seq text,
  mismatch int(10) default NULL,
  verificationCode int(10) default NULL,
  contaminated int(10) default NULL,
  sector varchar(20) default NULL,
  sectorRow int(10) default NULL,
  sectorCol int(10) default NULL,
  PRIMARY KEY  (spotId),
  KEY chipId (chipId)
) TYPE=MyISAM;

--
-- Table structure for table 'SpotPos'
--

CREATE TABLE SpotPos (
  spotId int(10) unsigned NOT NULL default '0',
  posId int(10) unsigned default NULL,
  locusId int(10) unsigned default NULL,
  KEY spotId (spotId),
  KEY orfId (locusId)
) TYPE=MyISAM;

--
-- Table structure for table 'SubExpAttrs'
--

CREATE TABLE SubExpAttrs (
  subExpId int(10) unsigned NOT NULL default '0',
  attr varchar(20) default NULL,
  value varchar(100) default NULL,
  KEY subExpId (subExpId)
) TYPE=MyISAM;

--
-- Table structure for table 'SubExpInfo'
--

CREATE TABLE SubExpInfo (
  subExpId int(10) unsigned NOT NULL auto_increment,
  expId int(10) unsigned NOT NULL default '0',
  biological int(5) unsigned default NULL,
  technical int(5) unsigned default NULL,
  file varchar(255) default NULL,
  controlDye varchar(255) default NULL,
  treatmentDye varchar(255) default NULL,
  imageId int(10) unsigned default NULL,
  PRIMARY KEY  (subExpId),
  KEY expId (expId)
) TYPE=MyISAM;

--
-- Table structure for table 'Synonym'
--

CREATE TABLE Synonym (
  name varchar(100) default NULL,
  locusId int(10) unsigned default NULL,
  version int(2) unsigned default '1',
  type int(2) unsigned default NULL,
  created date default NULL,
  source varchar(50) default NULL,
  KEY orfId (locusId),
  KEY orfId_version (locusId,version),
  KEY name (name),
  KEY type (type)
) TYPE=MyISAM;

--
-- Table structure for table 'TIGRInfo'
--

CREATE TABLE TIGRInfo (
  id varchar(16) default NULL,
  tigrId varchar(30) NOT NULL default '',
  type varchar(32) default NULL,
  roleId int(5) default NULL,
  geneSymbol varchar(10) default NULL,
  ec varchar(16) default NULL,
  definition longtext,
  PRIMARY KEY  (tigrId),
  KEY type (type),
  KEY roleId (roleId),
  KEY ec (ec)
) TYPE=MyISAM;

--
-- Table structure for table 'TaxDivision'
--

CREATE TABLE TaxDivision (
  divId int(10) unsigned NOT NULL default '0',
  code char(3) NOT NULL default '',
  name varchar(255) default NULL,
  comments text,
  PRIMARY KEY  (divId),
  UNIQUE KEY code (code)
) TYPE=MyISAM;

--
-- Table structure for table 'TaxName'
--

CREATE TABLE TaxName (
  taxonomyId int(10) unsigned NOT NULL default '0',
  name text NOT NULL,
  uniqueName text,
  class varchar(255) default NULL,
  KEY taxId (taxonomyId),
  KEY class (class)
) TYPE=MyISAM;

--
-- Table structure for table 'TaxNode'
--

CREATE TABLE TaxNode (
  taxonomyId int(10) unsigned NOT NULL default '0',
  parentId int(10) unsigned default NULL,
  rank varchar(50) default NULL,
  embl varchar(10) default NULL,
  divId int(3) unsigned NOT NULL default '0',
  flag1 int(1) unsigned NOT NULL default '0',
  flag2 int(1) unsigned NOT NULL default '0',
  flag3 int(1) unsigned NOT NULL default '0',
  flag4 int(1) unsigned NOT NULL default '0',
  flag5 int(1) unsigned NOT NULL default '0',
  flag6 int(1) unsigned NOT NULL default '0',
  flag7 int(1) unsigned NOT NULL default '0',
  comments varchar(255) default NULL,
  PRIMARY KEY  (taxonomyId),
  KEY parentId (parentId),
  KEY rank (rank),
  KEY divisionId (divId)
) TYPE=MyISAM;

--
-- Table structure for table 'TaxParentChild'
--

CREATE TABLE TaxParentChild (
  parentId int(10) unsigned NOT NULL default '0',
  childId int(10) unsigned NOT NULL default '0',
  nDistance int(10) unsigned NOT NULL default '0',
  UNIQUE KEY combined (parentId,childId),
  KEY parentId (parentId),
  KEY childId (childId)
) TYPE=MyISAM;

--
-- Table structure for table 'Taxonomy'
--

CREATE TABLE Taxonomy (
  taxonomyId int(10) NOT NULL default '0',
  name varchar(255) default NULL,
  placement int(10) default NULL,
  shortName varchar(100) default NULL,
  PRIMARY KEY  (taxonomyId)
) TYPE=MyISAM;

--
-- Table structure for table 'dbxref'
--

CREATE TABLE dbxref (
  id int(11) NOT NULL auto_increment,
  xref_key varchar(255) NOT NULL default '',
  xref_keytype varchar(32) default NULL,
  xref_dbname varchar(55) NOT NULL default '',
  xref_desc varchar(255) default NULL,
  PRIMARY KEY  (id),
  UNIQUE KEY dx0 (id),
  KEY dx1 (xref_dbname),
  KEY dx2 (xref_key),
  KEY dx3 (id,xref_dbname),
  KEY dx4 (id,xref_key,xref_dbname)
) TYPE=MyISAM;

--
-- Table structure for table 'graph_path'
--

CREATE TABLE graph_path (
  id int(11) NOT NULL auto_increment,
  term1_id int(11) NOT NULL default '0',
  term2_id int(11) NOT NULL default '0',
  distance int(11) NOT NULL default '0',
  PRIMARY KEY  (id),
  UNIQUE KEY graph_path0 (id),
  KEY graph_path1 (term1_id),
  KEY graph_path2 (term2_id),
  KEY graph_path3 (term1_id,term2_id),
  KEY graph_path4 (term1_id,distance)
) TYPE=MyISAM;

--
-- Table structure for table 'term'
--

CREATE TABLE term (
  id int(11) NOT NULL auto_increment,
  name varchar(255) NOT NULL default '',
  term_type varchar(55) NOT NULL default '',
  acc varchar(32) NOT NULL default '',
  is_obsolete int(11) NOT NULL default '0',
  is_root int(11) NOT NULL default '0',
  PRIMARY KEY  (id),
  UNIQUE KEY acc (acc),
  UNIQUE KEY t0 (id),
  KEY t1 (name),
  KEY t2 (term_type),
  KEY t3 (acc)
) TYPE=MyISAM;

--
-- Table structure for table 'term2term'
--

CREATE TABLE term2term (
  id int(11) NOT NULL auto_increment,
  relationship_type_id int(11) NOT NULL default '0',
  term1_id int(11) NOT NULL default '0',
  term2_id int(11) NOT NULL default '0',
  PRIMARY KEY  (id),
  UNIQUE KEY term1_id (term1_id,term2_id,relationship_type_id),
  KEY tt1 (term1_id),
  KEY tt2 (term2_id),
  KEY tt3 (term1_id,term2_id),
  KEY tt4 (relationship_type_id)
) TYPE=MyISAM;

--
-- Table structure for table 'term_dbxref'
--

CREATE TABLE term_dbxref (
  term_id int(11) NOT NULL default '0',
  dbxref_id int(11) NOT NULL default '0',
  is_for_definition int(11) NOT NULL default '0',
  UNIQUE KEY term_id (term_id,dbxref_id,is_for_definition),
  KEY tx0 (term_id),
  KEY tx1 (dbxref_id),
  KEY tx2 (term_id,dbxref_id)
) TYPE=MyISAM;

--
-- Table structure for table 'term_definition'
--

CREATE TABLE term_definition (
  term_id int(11) NOT NULL default '0',
  term_definition text NOT NULL,
  dbxref_id int(11) default NULL,
  term_comment mediumtext,
  reference varchar(255) default NULL,
  UNIQUE KEY term_id (term_id),
  KEY td1 (term_id)
) TYPE=MyISAM;

--
-- Table structure for table 'term_synonym'
--

CREATE TABLE term_synonym (
  term_id int(11) NOT NULL default '0',
  term_synonym varchar(255) default NULL,
  acc_synonym int(11) default NULL,
  UNIQUE KEY term_id (term_id,term_synonym),
  KEY ts1 (term_id),
  KEY ts2 (term_synonym),
  KEY ts3 (term_id,term_synonym)
) TYPE=MyISAM;

