package Experiment;
require Exporter;

use strict;
use Genome;
use GenomicsUtils;
use DBI;

our (@ISA, @EXPORT);

@ISA = qw(Exporter);
@EXPORT = qw();

{
sub new;
# Creates Experiment object (and all the subExperiment objects and the chip object)
# Parameters: expId => ID

# attribute fetching
sub expId($);
sub subExpIds($);		# returns list of ids
sub subExperimentsRef($);	# returns ref to list of objects
sub subExperiments($);		# returns list of objects
sub chipId($);
sub chip($);			# returns object
sub sampleTaxId($);
sub sampleGenome($);		# returns object
sub condition($);
sub controlInfo($);
sub treatmentInfo($);
sub pubMedId($);		# may be NULL (undef)
sub citation($);		# may be NULL
sub contact($);
sub email($);
sub comment($);

# Parameters: 'taxonomyId' => single value, or 'condition' => list
sub getExperiments;
sub getExpMap;			# returns a ref to taxonomyId->list of experiment objects

sub new{
    my %params = @_;
    my $expId = $params{'expId'};
    my $self = {};
    bless $self;

    my @columns = qw(expId chipId sampleTaxId condition controlInfo treatmentInfo pubMedId
		     citation contact email comment);
    # writes columnName_ values to $self
    GenomicsUtils::queryAttrs("ExpInfo","expId",$expId, \@columns, $self, "_");
    return $self;
}

sub expId($) {
    my $self = shift;
    return $self->{'expId_'};
}

sub subExpIds($) {
    my $self = shift;
    return map { $_->subExpId() } @{ $self->subExperiments() };
}

sub subExperimentsRef($) {
    my $self = shift;
    if (!defined $self->{'subExps'}) {
	$self->{'subExps'} = [];
	my @attrs = SubExperiment::columns();
	my $result = GenomicsUtils::queryTable(\@attrs,
					       "FROM SubExpInfo WHERE expId=" . $self->{'expId_'});
	foreach my $hash (@$result) {
	    my @params = map { $_ => $hash->{$_} } (keys %$hash);
	    push @{ $self->{'subExps'} }, SubExperiment::new(@params);
	}
    }
    return $self->{'subExps'};
}

sub subExperiments($) {
    return @{ shift->subExperimentsRef() };
}

sub chipId($) {
    my $self = shift;
    return $self->{'chipId_'};
}

sub chip($) {
    my $self = shift;
    return Chip::new('chipId' => $self->{'chipId_'});
}

sub sampleTaxId($) {
    my $self = shift;
    return $self->{'sampleTaxId_'};
}

sub sampleGenome($) {
    my $self = shift;
    return Genome::new('taxonomyId' => $self->{'sampleTaxId_'});
}

sub condition($) {
    my $self = shift;
    return $self->{'condition_'};
}

sub controlInfo($) {
    my $self = shift;
    return $self->{'controlInfo_'};
}

sub treatmentInfo($) {
    my $self = shift;
    return $self->{'treatmentInfo_'};
}

sub pubMedId($) {
    my $self = shift;
    return $self->{'pubMedId_'};
}

sub citation($) {
    my $self = shift;
    return $self->{'citation_'};
}

sub contact($) {
    my $self = shift;
    return $self->{'contact_'};
}

sub email($) {
    my $self = shift;
    return $self->{'email_'};
}

sub comment($) {
    my $self = shift;
    return $self->{'comment_'};
}

sub getExperiments {
    my $hashRef = getExpMap(@_);
    my @exps = ();
    map { push @exps, @{ $_ } } values %{ $hashRef };
    return(@exps);
}

sub getExpMap {
    my %params = @_;
    my @where = ();

    if (exists $params{'taxonomyId'}) {
	push @where, "sampleTaxId=" . $params{'taxonomyId'};
    }
    if (exists $params{'condition'}) {
	my @conditions = map { "'" . $_ . "'" } @{ $params{'condition'} };
	push @where, "condition IN (" . join(',', @conditions) . ")";
    }
    my $where = (scalar @where > 0 ? "WHERE " . join(" AND ", @where) : "");
    my $hashQ = GenomicsUtils::queryHashList("SELECT sampleTaxId,expId from ExpInfo $where");
    my $hashRef = {};
    foreach my $tax (keys %{ $hashQ }) {
	$hashRef->{$tax} = [];
	foreach my $expId (@{ $hashQ->{$tax} }) {
	    push @{ $hashRef->{$tax} }, Experiment::new('expId' => $expId);
	}
    }
    return($hashRef);
}

}

return 1;
