reload <- function() {
	source("Genomics/util/utils.R");
	source("workbook/UnsupervisedOperons/plots.R");
}

LoadPlots <- function(predictOp=FALSE,buildOrfPairs=FALSE,buildAOC=FALSE,nfold=100,...) {
	d <- list();
	d$opAll <- read.delim("operons/out/gncAllGenomes.stats");
	#opK <- d$opAll$genome != 181661; d$opAll <- select(d$opAll, opK);
	d$opAllNaive <- read.delim("operons/out/gncAllGenomes.statsNaivefOperon");
	d$opAllMI <- read.delim("operons/out/gncAllGenomes.withMI");
	genomeNames <- read.delim("operons/out/genomeNames")
	d$opAll$nameShort <- genomeNames$shortName[findMatch(d$opAll$genome,genomeNames$taxonomyId)$index];
	d$opAll$source <- genomeNames$source[findMatch(d$opAll$genome,genomeNames$taxonomyId)$index];

	if (predictOp) {
		d$hbOp <- OperonsPredict(64091,"Halobacterium",...);
		cat("done hb\n");

		d$ecOp <- OperonsPredict(83333,"E.coli K12",...);

		d$bsOp <- OperonsPredict(1423,"B.subtilis",...);

		d$soOp <- OperonsPredict(211586,"Shewanella oneidensis",...);
		d$ctOp <- OperonsPredict(813,"Chlamydia trachomatis",...);
		d$scOp <- OperonsPredict(1148,"Synechocystis PCC 6803",...);
		d$hpOp <- OperonsPredict(85962,"Helicobacter pylori 26695",...);
	} else {
		d$ecOp <- ReadOp(select(d$opAll,d$opAll$genome==83333),...);
		d$bsOp <- ReadOp(select(d$opAll,d$opAll$genome== 1423),...);
		d$hpOp <- ReadOp(select(d$opAll,d$opAll$genome==85962),...);
		d$ctOp <- ReadOp(select(d$opAll,d$opAll$genome==  813),...);
		d$scOp <- ReadOp(select(d$opAll,d$opAll$genome== 1148),...);
		d$hbOp <- ReadOp(select(d$opAll,d$opAll$genome==64091),...);
	}
	bsd <- OperonsData(1423,"B.subtilis");
	d$bsOpNaive <- OperonsPredict(1423,"B.subtilis",usefOperonNaive=TRUE,anova=FALSE,data=bsd,...);
	bsd$fOperon <- .5*d$bsOp$fMoreno/d$ecOp$fMoreno;
	d$bsOpMoreno <- OperonsPredict(1423,"B.subtilis",data=bsd,anova=FALSE,...);

	ecd <- OperonsData(83333,"E.coli K12");
	ecd$fOperon <- .5;
	d$ecOpMoreno <- OperonsPredict(83333,"E.coli K12",data=ecd,anova=FALSE,...);
	cat("done ops\n");

	d$ecOp$name <- "E.coli K12";
	d$bsOp$name <- "B.subtilis";
	d$hpOp$name <- "H.pylori 26695";
	d$ctOp$name <- "C.trachomatis";
	d$scOp$name <- "Synechocystis PCC 6803";
	d$hbOp$name <- "Halobacterium NRC-1";

	if(buildAOC) {
		d$ecAOC <- OperonsAOC(d$ecOp,nfold=nfold);
		d$bsAOC <- OperonsAOC(d$bsOp,nfold=nfold);
		writeDelim(d$ecAOC$predsS,"operons/out/ecAOCpredsS");
		writeDelim(d$ecAOC$predsD,"operons/out/ecAOCpredsD");
		writeDelim(cbind(d$ecOp$preds[,c("Gene1","Gene2")],known=d$ecAOC$known,
				pTrained=d$ecAOC$pTrained,
				pTrainedD=d$ecAOC$pTrainedD,
				pTrainedLogistic=d$ecAOC$pTrainedLogistic,
				pTrainedCAI=d$ecAOC$pTrainedCAI),
				"operons/out/ecAOCall");
		writeDelim(d$bsAOC$predsS,"operons/out/bsAOCpredsS");
		writeDelim(d$bsAOC$predsD,"operons/out/bsAOCpredsD");
		writeDelim(cbind(d$bsOp$preds[,c("Gene1","Gene2")],known=d$bsAOC$known,
				pTrained=d$bsAOC$pTrained,
				pTrainedD=d$bsAOC$pTrainedD,
				pTrainedLogistic=d$bsAOC$pTrainedLogistic,
				pTrainedCAI=d$bsAOC$pTrainedCAI),
				"operons/out/bsAOCall");
	} else {
		d$ecAOC <- c(as.list(read.delim("operons/out/ecAOCall")),
				list(predsS=read.delim("operons/out/ecAOCpredsS"),
				predsD=read.delim("operons/out/ecAOCpredsD")));
		d$ecAOC$predsS$bOp <- codes(d$ecAOC$predsS$bOp)==2;
		d$ecAOC$predsS$bOp1 <- codes(d$ecAOC$predsS$bOp1)==2;
		d$ecAOC$predsD$bOp <- codes(d$ecAOC$predsD$bOp)==2;
		d$ecAOC$predsD$bOp1 <- codes(d$ecAOC$predsD$bOp1)==2;

		d$ecAOC$aocTrained <- AOC(d$ecAOC$predsS$pTrained,d$ecAOC$predsD$pTrained);
		d$ecAOC$aocUntrained <- AOC(d$ecAOC$predsS$pOp,d$ecAOC$predsD$pOp);
		d$ecAOC$aocUntrained1 <- AOC(d$ecAOC$predsS$pOp1,d$ecAOC$predsD$pOp1);
		d$ecAOC$aocDistance <- AOC(d$ecAOC$predsS$pSameD,d$ecAOC$predsD$pSameD);
		d$ecAOC$aocPSame <- AOC(d$ecAOC$predsS$pSame,d$ecAOC$predsD$pSame);

		d$bsAOC <- c(as.list(read.delim("operons/out/bsAOCall")),
				list(predsS=read.delim("operons/out/bsAOCpredsS"),
				predsD=read.delim("operons/out/bsAOCpredsD")));
		d$bsAOC$predsS$bOp <- codes(d$bsAOC$predsS$bOp)==2;
		d$bsAOC$predsS$bOp1 <- codes(d$bsAOC$predsS$bOp1)==2;
		d$bsAOC$predsD$bOp <- codes(d$bsAOC$predsD$bOp)==2;
		d$bsAOC$predsD$bOp1 <- codes(d$bsAOC$predsD$bOp1)==2;

		d$bsAOC$aocTrained <- AOC(d$bsAOC$predsS$pTrained,d$bsAOC$predsD$pTrained);
		d$bsAOC$aocUntrained <- AOC(d$bsAOC$predsS$pOp,d$bsAOC$predsD$pOp);
		d$bsAOC$aocUntrained1 <- AOC(d$bsAOC$predsS$pOp1,d$bsAOC$predsD$pOp1);
		d$bsAOC$aocDistance <- AOC(d$bsAOC$predsS$pSameD,d$bsAOC$predsD$pSameD);
		d$bsAOC$aocPSame <- AOC(d$bsAOC$predsS$pSame,d$bsAOC$predsD$pSame);
	}
	cat("done AOC\n");

	d$scCors <- ReadUarrayOpCors("sc",scOp,buildOrfPairs=buildOrfPairs);	cat("finished scCors\n");
	d$ctCors <- ReadUarrayOpCors("ct",ctOp,buildOrfPairs=buildOrfPairs);	cat("finished ctCors\n");
	d$hpCors <- ReadUarrayOpCors("hp",hpOp,buildOrfPairs=buildOrfPairs);	cat("finished hpCors\n");
	d$bsCors <- ReadUarrayOpCors("bs",bsOp,buildOrfPairs=buildOrfPairs);	cat("finished bsCors\n");
	d$ecCors <- ReadUarrayOpCors("ec",ecOp,buildOrfPairs=buildOrfPairs);	cat("finished ecCors\n");

	d$hbCors <- cbind(read.delim("operons/data/hbCors"),sumD=0,sumD1=0);
	d$hbCors$orfPairs <- sqrt(abs(d$hbCors$SignedRSqr))*sign(d$hbCors$SignedRSqr);

	#	hbGene1 <- rbind(read.delim("downloads/ericAlmOp.pred12"),
	#				read.delim("downloads/ericAlmOp.pred21"))[,c("Gene1","SignedRSqr")];
	#	hbMatch <- findMatch(d$hbOp$preds$Gene1,hbGene1$Gene1)$index;
	#	d$hbP2 <- cbind(select(d$hbOp$preds,hbMatch>0),
	#				SignedRSqr=hbGene1$SignedRSqr[select(hbMatch,hbMatch>0)]);
	#	d$hbP2$orfPairs <- sqrt(abs(d$hbP2$SignedRSqr))*sign(d$hbP2$SignedRSqr);

	# same-strand pairs which have uarray evidence
	d$ecP2 <- cbind(d$ecOp$preds,select(d$ecCors,d$ecOp$gcSC$same),
				pTrained=d$ecAOC$pTrained,
				pTrainedLogistic=d$ecAOC$pTrainedLogistic,
				pTrainedD=d$ecAOC$pTrainedD,
				pTrainedCAI=d$ecAOC$pTrainedCAI,
				known=d$ecAOC$known);
	d$bsP2 <- cbind(d$bsOp$preds,select(d$bsCors,d$bsOp$gcSC$same),
				pTrained=d$bsAOC$pTrained,
				pTrainedLogistic=d$bsAOC$pTrainedLogistic,
				pTrainedD=d$bsAOC$pTrainedD,
				pTrainedCAI=d$bsAOC$pTrainedCAI,
				known=d$bsAOC$known);
	d$hpP2 <- cbind(d$hpOp$preds,select(d$hpCors,d$hpOp$gcSC$same));
	d$ctP2 <- cbind(d$ctOp$preds,select(d$ctCors,d$ctOp$gcSC$same));
	d$scP2 <- cbind(d$scOp$preds,select(d$scCors,d$scOp$gcSC$same));
	d$hbP2 <- cbind(d$hbOp$preds,select(d$hbCors,d$hbOp$gcSC$same));

	d$ecP2 <- select(d$ecP2, !is.na(d$ecP2$orfPairs));
	d$bsP2 <- select(d$bsP2, !is.na(d$bsP2$orfPairs));
	d$hpP2 <- select(d$hpP2, !is.na(d$hpP2$orfPairs));
	d$ctP2 <- select(d$ctP2, !is.na(d$ctP2$orfPairs));
	d$scP2 <- select(d$scP2, !is.na(d$scP2$orfPairs));
	d$hbP2 <- select(d$hbP2, !is.na(d$hbP2$orfPairs));

	cat("finished P2s except hb\n");

	# These can be recreated by calls such as
	# scRatios <- read.delim("operons/data/scRatios");
	# scSEs <- read.delim("operons/data/scSEs");
	# scSop <- SingleUarrayOpTest(scOp$preds,scRatios,select(2:length(scRatios),scSEs$condition!="Genomic"));
	# scSop$cond <- scSEs[scSop$dim-1,"condition"]
	# writeDelim(scSop,"operons/out/scSingleArray")

	d$ecSop <- read.delim("operons/out/ecSingleArray");
	d$bsSop <- read.delim("operons/out/bsSingleArray");
	d$hpSop <- read.delim("operons/out/hpSingleArray");
	d$ctSop <- read.delim("operons/out/ctSingleArray");
	d$scSop <- read.delim("operons/out/scSingleArray");

	d$ecBBFDistT <- BinnedBinaryFit2(d$ecAOC$predsS$Sep,d$ecAOC$predsD$Sep);
	d$bsBBFDistT <- BinnedBinaryFit2(d$bsAOC$predsS$Sep,d$bsAOC$predsD$Sep);

	d$gncAll <- read.delim("operons/out/gncAllGenomesBN");

	# These were created with UarrayOrfPairSubsampleAll, e.g.:
	# ecAcc <- UarrayOrfPairSubsampleAll("ec",ecOp); writeDelim(ecAcc,"operons/out/ecSubsampled");
	# and reflect jackknife on microarrays grouped by condition ("skip" is the one left out)

	d$ecAcc <- read.delim("operons/out/ecSubsampled");
	d$bsAcc <- read.delim("operons/out/bsSubsampled");
	d$hpAcc <- read.delim("operons/out/hpSubsampled");
	d$ctAcc <- read.delim("operons/out/ctSubsampled");
	d$scAcc <- read.delim("operons/out/scSubsampled");

	# These are resampled estimates of error, and were created with e.g.
	#ecAcc2 <- AccResampleGenes(ecOp,ecCors)
	d$ecAcc2 <- read.delim("operons/out/ecAcc2");
	d$bsAcc2 <- read.delim("operons/out/bsAcc2");
	d$hpAcc2 <- read.delim("operons/out/hpAcc2");
	d$ctAcc2 <- read.delim("operons/out/ctAcc2");
	d$scAcc2 <- read.delim("operons/out/scAcc2");
	d$hbAcc2 <- read.delim("operons/out/hbAcc2");

	cat("done d",names(d),"\n");
	for (v in names(d)) {
		assign(v, d[[v]], env=.GlobalEnv);
	}
}

ReadOp <- function(row,gncDir="operons/out",prefix="gnc",...) {
	op <- row;
	op$gcSC <- read.delim(paste(gncDir,"/",prefix,op$genome,".gc",sep=""));
	op$preds <- read.delim(paste(gncDir,"/",prefix,op$genome,".pred",sep=""));
	op$bbfD <- list();
	op$bbfD$groups <- read.delim(paste(gncDir,"/",prefix,op$genome,".sep",sep=""));
	op$bbfCAI <- list();
	op$bbfCAI$groups <- read.delim(paste(gncDir,"/",prefix,op$genome,".caidxdy",sep=""));
	op$without <- c("without","gcSC","preds","bbfD","bbfCAI","vars");
	op$gcSC$same <- codes(op$gcSC$same) == 2;
        op$preds$bOp <- codes(op$preds$bOp) == 2;
        op$preds$bOp1 <- codes(op$preds$bOp1) == 2;
	op$preds$forward <- codes(op$preds$forward) == 2;
	op$gcSC$forward <- codes(op$gcSC$forward) == 2;
	op$formula <- levels(op$formula)[op$formula];
	op$vars <- c("GNScore","GNMinus","GNWithin","GNAll","HK","MI","CAIdxdyR", "CAIdsqrR");
 	return(op);
}

addEc <- function(preds) {
	preds$pOpDistanceEc <- BBFPredict(preds$Sep, ecBBFDistT)$p;
	preds$pOpEc <- combineP(preds$pOpLogistic,preds$pOpDistanceEc);
	return(preds);
}

MakePlots <- function(plots=c("ecPlot","bsPlot","ecbsPlots",
				"loessPlot","opAllPlot","opAllPlotSep","ExpVsUnsupervisedPlots",
				"POpDistanceVSfOperon","LeadingVSLaggingPlots","AssumptionPlot",
				"SingleUarrayPlot","fOperonVsUarrayPlots","JackknifePlot",
				"CAIplots","SepVsUarrayPlots"),jpg=TRUE,ps=TRUE) {
	relHeights <- list(ExpVsUnsupervisedPlots=.5, LeadingVSLaggingPlots=1/2,ecbsPlots=1,loessPlot=3/2,
				JackknifePlot=1/3,
				AssumptionPlot=1/3,POpDistanceVSfOperon=2/3,SingleUarrayPlot=1,
				fOperonVsUarrayPlots=2/3,SepVsUarrayPlots=1);
	for (name in plots) {
		relHeight <- if (is.null(relHeights[[name]])) 1 else relHeights[[name]];
		cat("Writing workbook/UnsupervisedOperons/",name,".jpg","\n",sep="");
		if(jpg) {
		jpeg(paste("workbook/UnsupervisedOperons/",name,".jpg",sep=""),width=800,height=600*relHeight);
		eval(parse(text=paste(name,"()",sep="")));
		dev.off();
		}
		if (ps) {
		cat("Writing workbook/UnsupervisedOperons/",name,".ps","\n",sep="");
		postscript(paste("workbook/UnsupervisedOperons/",name,".ps",sep=""),horizontal=FALSE,
				width=12,height=9*relHeight);
		eval(parse(text=paste(name,"()",sep="")));
		dev.off();
		}
	}
}

MakeTables <- function(tables=c("ecTableComp","bsTableComp",
				"ecTable","bsTable","hpTable","ctTable","hbTable","scTable",
		"genomeTable","uarrayTable")) {
	on.exit(sink(NULL));
	for (name in tables) {
		inc <- paste("workbook/UnsupervisedOperons/",name,".inc",sep="");
		cat("Writing",inc,"\n");
		sink(file=inc);
		cat("%","Running",name,"\n");
		eval(parse(text=paste(name,"()",sep="")));
		sink(NULL);
	}
	cat("Wrote table(s) ",tables,"\n");
}

ecbsPlots <- function(points=FALSE,sep=TRUE,bbf=FALSE,aoc=TRUE,uarray=TRUE) {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(mfrow=c(countif(c(sep,bbf,aoc,uarray)),2),mar=c(5,4,.5,1),mgp=c(2,.75,0),cex=.9,cex.axis=1.2);

	if (sep) { ecPlot("sep",title=FALSE); bsPlot("sep",title=FALSE); }
	if (bbf) { ecPlot("bbf",title=FALSE); bsPlot("bbf",title=FALSE); }
	if (aoc) { ecPlot("aoc",points=points,title=FALSE); bsPlot("aoc",points=points,title=FALSE); }
	if (uarray) { ecPlot("uarray",title=FALSE); bsPlot("uarray",title=FALSE); }
}

ecPlot <- function(plots=NULL,...) { PlotOpAOC(ecOp,ecAOC,ecP2,ecBBFDistT,plots=plots,...); }
bsPlot <- function(plots=NULL,...) { PlotOpAOC(bsOp,bsAOC,bsP2,bsBBFDistT,plots=plots,...); }

PlotOpAOC <- function(op, aoc, P2, bbfDistT, plots=NULL, points=FALSE, title=TRUE) {
	if (is.null(plots)) plots <- c("sep","bbf","aoc","uarray");
	if (length(plots)>1) {
		old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
		par(mfrow=c(2,2),cex.axis=1.5);
	}
for (plot in plots) {
if (plot=="sep") {
	if(is.null(aoc)) {
		CompareBBFGroups(list(unsupervised=op$bbfD$groups),
				main=if(title) "Distance Model" else "",
				xlab=paste("Intergenic Distance", "  (", op$name, ")", sep=""),
				ylab="ln(likelihood ratio)",
				xlim=c(-40,300),ylim=c(-1,1)*3.5,legend=FALSE);
	} else {
		CompareBBFGroups(list(unsupervised=op$bbfD$groups,trained=bbfDistT$groups),
				main=if(title) "Distance Model" else "",
				xlab=paste("Intergenic Distance", "  (", op$name, ")", sep=""),
				ylab="ln(likelihood ratio)",ltyH=c(2,1),lty=c(0,0),
				col=c(2,1),pch=c(30,30),
				xlim=c(-40,300),ylim=c(-1,1)*3.5,legend=FALSE);
		lines(sort(op$preds$Sep),logodds(BBFPredict(sort(op$preds$Sep),bbfDistT)$p),
			col="grey",lwd=1,lty=1);
	}
	sortedLines(op$preds$Sep,logodds(op$preds$pSameD),col=2,lwd=2,lty=1);
	hline(0);
	if (!is.null(aoc)) {
		legend(100,3.5,c("Trained","    (bins)",
				"Untrained","    (bins)"),
				col=c("grey","black","red","red"),pch=c(30,30,30,30),lty=c(1,1,1,2),lwd=c(1,1,2,1));
	}
}
if (plot=="bbf") {
	bbfD <- BinnedBinaryFit2(aoc$predsS$pOpDistance,aoc$predsD$pOpDistance,plot=TRUE,
				main=if(title) "P(Operon) vs. Experimental Operons" else "",
				xlim=0:1,ylim=0:1,
				xlab=paste("Unsupervised P(Operon)", "  (", op$name, ")", sep=""),
				ylab="%Exp. operons");
	eqline();
}
if (plot=="aoc") {
	plot(1-aoc$aocUntrained$spec,aoc$aocUntrained$sens, type="l", lty=1, col=1,
			xlab = paste("False Positives  (",op$name,")", sep=""),
			ylab = paste("True Positives"),
			main = if (title) "Receiver Operating Curve" else "");
	if(points) points(countPerIf(aoc$predsD$bOp),countPerIf(aoc$predsS$bOp), col=1, pch=2);

	lines(1-aoc$aocTrained$spec, aoc$aocTrained$sens, lty=8, col=6, lwd=2);
	if(points) points(1-countPerIf(aoc$predsD$pTrained<.5), countPerIf(aoc$predsS$pTrained>=.5), col=6,pch=8);

	lines(1-aoc$aocDistance$spec, aoc$aocDistance$sens, lty=6, col=4);
	if(points) points(1-countPerIf(aoc$predsD$pSameD<.5), countPerIf(aoc$predsS$pSameD>=.5), col=4,pch=20);

	lines(1-aoc$aocPSame$spec, aoc$aocPSame$sens, lty=7, col=5);
	if(points) points(1-countPerIf(aoc$predsD$pOpLogistic<.5), countPerIf(aoc$predsS$pOpLogistic>=.5), col=5,pch=20);

	legend(1.05,.6,c("Trained", "Untrained", "    distance-only", "    comparative-only"),
		pch=if(points) c(8,2,20,30) else rep(30,4), col=c(6,1,4,5), lty=c(8,1,6,7), lwd=c(2,1,1,1), xjust=1);
}
if(plot=="uarray") {
	ExpVsUnsupervisedPlot(P2,op$name,title=title);
}
if(plot=="uarrayOld"){ # obsolete
	pHigh <- .99; pLow <- .03;
	comp <- list(exp=select(P2$orfPairs,P2$known==1),
			expNot=select(P2$orfPairs,P2$known==-1),
			pred=select(P2$orfPairs,P2$bOp),
			predNot=select(P2$orfPairs,!P2$bOp),
			confOp=select(P2$orfPairs,P2$pOp>pHigh),
			confNop=select(P2$orfPairs,P2$pOp<pLow));
	counts <- sapply(comp, length);
	labels <- c("Experimental Op. Pairs",
					"Exp. !Op. Pairs",
					"Pred. Op.: Unsupervised p > .5",
					"Pred. !Op.: Unsupervised p <= .5",
					paste("Confident Pred. Op: p > ", pHigh ,sep=""),
					paste("Confident Pred. !Op: p < ", pLow, sep=""));
	for (i in 1:length(labels)) { labels[i] <- paste(labels[i], "  (", counts[i], ")", sep=""); }
	CompareHistogramList(comp, labels=labels, breaks=c(-5:5)/5,
				xlab=paste("Microarray Similarity (Pearson r) for ",op$name,sep=""),
				ylab="Proportion of pairs",
				main=if(title) "Microarray vs. Operons" else "",
				ylim=c(0,.5),
				legendX=-.5,legendY=.5,cex=.9);
}
	}
}

loessPlot <- function(n=NULL,useDensity=NULL,bigMar=FALSE) {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(mfrow=c(3,2));
	par(cex=1.25);
	par(mar=rep(ifelse(bigMar,.5,.25),4),oma=rep(2,4),
			xaxt="n",yaxt="n", bty="n");
	par(xaxs="i",yaxs="i");
	par(tcl=-.2);
	xlabels <- c(" 0","0.25","0.5","0.75","1 ");
	xlabels2 <- c(" 0","0.125","0.25","0.125","0 ");
	ylabels=c(" -1","-0.5","0","0.5","1 ");
	ylabels2=c(" 0","0.25","0.5","0.75","1 ");

	loessPlotGenome(ecOp,ecCors,ecP2,n=n,useDensity=useDensity,legend=FALSE);
	axis(3,labels=xlabels2,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(0,1,.25),line=-.8);
	axis(2,labels=ylabels,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(-1,1,.5),line=-.8);

	loessPlotGenome(bsOp,bsCors,bsP2,n=n,useDensity=useDensity,legend=FALSE);
	axis(3,labels=xlabels2,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(0,1,.25),line=-.8);
	axis(4,labels=ylabels2,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(-1,1,.5),line=-.8);

	loessPlotGenome(hpOp,hpCors,hpP2,n=n,useDensity=useDensity,legend=1);
	axis(2,labels=ylabels,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(-1,1,.5),line=-.8);

	loessPlotGenome(ctOp,ctCors,ctP2,n=n,useDensity=useDensity,legend=2);
	axis(4,labels=ylabels2,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(-1,1,.5),line=-.8);

	loessPlotGenome(scOp,scCors,scP2,n=n,useDensity=useDensity);
	axis(1,labels=xlabels,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(0,1,.25),line=-.8);
	axis(2,labels=ylabels,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(-1,1,.5),line=-.8);

	loessPlotGenome(hbOp,hbCors,hbP2,n=n,useDensity=useDensity);
	axis(1,labels=xlabels,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(0,1,.25),line=-.8);
	axis(4,labels=ylabels2,tick=FALSE,outer=FALSE,xaxt="s",yaxt="s",at=seq(-1,1,.5),line=-.8);

	mtext("p(Operon)",side=1,outer=TRUE,line=.75);
	mtext("Pearson correlation in microarrays (r)",side=2,outer=TRUE,line=.75);
	mtext("Proportion",side=3,outer=TRUE,line=.75);
	mtext("Proportion",side=4,outer=TRUE,line=.75);
}

loessPlotGenome <- function(op,cors,P2,legend=FALSE,n=4,lrHist=TRUE,bDens=FALSE,useDensity=TRUE,known=FALSE) {
	on <- select(P2$orfPairs,P2$bOp);
	off <- select(P2$orfPairs,!P2$bOp)
	ks <- ks.test(off,on);

	breaksP <- (0:20)/20;
	hp <- hist(P2$pOp,breaks=breaksP,plot=FALSE);

	plot(c(0,1),c(-1,1),col=-1,xlab="",ylab="");
	rect(breaksP[1:(length(breaksP)-1)],-1,breaksP[2:length(breaksP)],-1+2*hp$counts/length(P2$pOp),
			col="grey");

	for(i in c(1,3)) { axis(i,labels=FALSE, tick=TRUE, outer=FALSE,xaxt="s",yaxt="s",at=seq(0,1,.25),line=0); }
	for(i in c(2,4)) { axis(i,labels=FALSE, tick=TRUE, outer=FALSE,xaxt="s",yaxt="s",at=seq(-1,1,.5),line=0); }
	rect(0,-1,1,1);
	#lines(c(0,.5),rep(median(off),2),lty=1,col="grey",lwd=2);
	#lines(c(1,.5),rep(median(on),2),lty=1,col="grey",lwd=2);

	if (bDens) {
		densityP <- density(P2$pOp,from=0,to=1);
		densityP$y <- densityP$y/sum(densityP$y);
		lines(densityP$x,-1+densityP$y*2*length(densityP$x)/(length(breaksP)-1));
	}
	loessP <- loess(orfPairs~rank(pOp),P2);
	lines(sort(P2$pOp),predict(loessP,newdata=1:length(P2$pOp)),col="red",lwd=2);

	breaks <- (-20:20)/20;
	if (lrHist) {
		hon <- hist(on,breaks=breaks,plot=FALSE);
		hoff <- hist(off,breaks=breaks,plot=FALSE);
		rect(0, breaks[1:(length(breaks)-1)],2*hoff$counts/length(off),breaks[2:length(breaks)]);
		rect(1-2*hon$counts/length(on), breaks[1:(length(breaks)-1)],1,breaks[2:length(breaks)]);
	}
	if(known) {
		P2k <- select(P2,P2$known != 0);
		loessK <- loess(orfPairs~rank(pOp),P2k,span=.5);
		lines(sort(P2k$pOp),predict(loessK)[order(P2k$pOp)],lty=2,col="darkgrey",lwd=2);
		lines(c(0,.5),rep(median(select(P2k$orfPairs,!P2k$bOp)),2),lty=3,lwd=1,col="black");
		lines(c(1,.5),rep(median(select(P2k$orfPairs,P2k$bOp)),2),lty=3,lwd=1,col="black");
	}

	F <- fOperonVsUarray(op,cors,n=n,useDensity=useDensity);
	modelOp <- summary(lm(I(op-controlO)~I(highO-controlO)+0,F));
	modelNop <- summary(lm(I(nop-highN)~I(controlN-highN)+0,F));
	accT <- modelOp$coefficients[1,1];
	accF <- modelNop$coefficients[1,1];
	mult <- 2*length(F$x)/(length(breaks)-1);
	#lines(mult*(accF*F$controlN+(1-accF)*F$highN),F$x,col="darkgrey",lwd=2);
	lines(mult*F$controlN,F$x,col="orange",lty=5,lwd=1);
	lines(mult*F$nop,F$x);
	#lines(1-mult*(accT*F$highO+(1-accT)*F$controlO),F$x,col="darkgrey",lwd=2);
	lines(1-mult*F$highO,F$x,col="green",lty=5,lwd=1);
	lines(1-mult*F$op,F$x);
	if (legend==1) legend(.13, 1.05,
			c("r, pred. op. or not",
				"r, confident pred.",
				"r, opposing-strand",
				"p(Operon)"),
			lty=c(1,5,5,1),
			pch=c(22,-1,-1,15),col=c("black","green","orange","grey"),lwd=c(1,1,2,2,1),cex=.9);
	if (legend==2) {
		if (known) { legend(.13, 1.05,
			c("fit of p vs. r (all)","    (exp. subset)"), ## "median r","    (exp. subset)"),
			lty=c(1,2),col=c("red","darkgrey"),lwd=c(2,2),cex=.9);
			#lty=c(1,2,1,3),col=c("red","darkgrey","grey","black"),lwd=c(2,2,2,1),cex=.9);
		}
		else { legend(.13, 1.05, c("fit of p vs. r"), lty=1, col="red", lwd=2, cex=.9); }
	}

	f <- function(x) { return(format(x,digits=2,nsmall=2)); }
	s <- c(op$name,
		paste("D = ",f(ks$statistic),"  (p ",
			(if(ks$p.value==0) "< 1e-13" else paste("=",f(ks$p.value))),")",sep=""),
		"accuracy from microarrays:",
		paste("    pred. op.: ", f(accT),
			" (vs. ", f(mean(select(P2$pOp,P2$bOp))), ")", sep=""),
		paste("    pred. not: ",f(accF),
			" (vs. ", f(1-mean(select(P2$pOp,!P2$bOp))),")", sep="")
		);
	lx <- .15; ly <- -.25; ww <- max(strwidth(s))+strwidth(" "); hh <- 1.3*strheight(s[1]);
	text(rep(lx,5),ly+hh*seq(2,-2,-1),s,adj=0);
	cat("Genome",op$name,"estimated accuracy: op",f(accT),"nop",f(accF),
			"using n=",n,
			"op fit",f(sum(abs(accT*F$highO+(1-accF)*F$controlO-F$op))),
			"nop",f(sum(abs((1-accF)*F$highN+accF*F$controlN-F$nop))),"\n");
}

opAllPlotSep <- function() { opAllPlot("dotplots"); }

opAllPlot <- function(plots=c("rLogistic","Moreno","vsRaw","sig")) {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	if (length(plots) == 4) { par(mfrow=c(2,2)); }
	else { if( length(plots) != 1) par(mfrow=c(3,2)); }

	colors <- rep(8,length(opAll$name));
	changes <- c(ecOp$genome,bsOp$genome,hpOp$genome,ctOp$genome,scOp$genome,hbOp$genome);
	for (i in 1:length(changes)) {
		colors[select(1:length(opAll$genome),opAll$genome==changes[i])] <- i+1;
	}
	pch <- ifelse(colors==8,1,colors);
	colors <- ifelse(colors==8,"grey",colors);

for (plot in plots) {
    if (plot=="rLogistic"){
	plot(opAll$rLogistic,opAll$fSameGN-opAll$fControlGN,
			xlab="Effectiveness of logistic regression (Spearman)",
			ylab="P(GNScore>0 | Same) - P(GNScore>0 | !Same)",
			main="A.  Comparative Model of Strands",
			col=colors,pch=pch, xlim=c(0,.5), ylim=c(0,.5));
	legend(.0,.5,c("E.coli","B.subtilis","H.pylori","C.trachomatis","Synechocystis","Halobacterium"),
		col=2:7, pch=2:7,xjust=0,yjust=1, cex=.9);
	#points(opAll$rLogistic,opAll$fSameGN-opAll$fControlGN,col=2,pch=2);
	#linePairs(opAll$rLogistic,opAll$fSameGN, opAll$rLogistic, opAll$fControlGN,col="grey");
   }
   if (plot=="Moreno") {
	plot(opAll$fOperon, opAll$fMoreno*.5/ecOp$fMoreno,
			xlab="P(Operon | Same) -- our method",
			ylab="P(Operon | Same) from E.coli model",
			main="B.  Estimating #Operons", col=colors,pch=pch);
	eqline(col=1);
   }
   if (plot=="dotplots") {
	namesSep <- c(-10, -3, 0, 25, 50, 100, 150, 200, 300);
	namedSep <- sapply(namesSep, function(x) { paste("pOpDistanceSep",ifelse(x<0,"Minus",""),abs(x),sep=""); });
	seps <- opAll[,namedSep];
	names(seps) <- ifelse(namesSep<=0,namesSep-1,namesSep);
	groupsSep <- combineColumns(seps);
	plot(groupsSep$type,groupsSep$x, xlab=paste("Intergenic Distance (d)"),
					ylab="Distance model p(d)",cex.lab=1.5);
	n <- length(namesSep)-1;
	#for (i in 1:n) { linePairs(i,seps[,i],i+1,seps[,i+1], col="grey"); }
	genomes <- c(ecOp$genome, bsOp$genome, hpOp$genome, ctOp$genome, scOp$genome, hbOp$genome,198094);
	gNames <- c("E.coli","B.subtilis","H.pylori","C.trachomatis", "Synechocystis","Halobacterium","B.anthracis");
	gCol <- c(1,3,4,5,6,7,"orange"); gLty <- c(2,1,4,5,6,7,1); gLwd <- c(2,2,2,2,2,2,2);
	for (g in 1:length(genomes)) {
		for (i in 1:n) {
			x <- select(1:length(opAll$genome),opAll$genome==genomes[g]);
			linePairs(i,seps[x,i],i+1,seps[x,i+1], col=gCol[g], lty=gLty[g],lwd=gLwd[g]);
		}
	}
	legend(.4,0,gNames,col=gCol,lty=gLty,lwd=gLwd,yjust=0,cex=1.2);
    }
    if (plot=="vsRaw") {
	plot(opAll$GNScoreVSSep,opAll$pOpLogisticVSpOpDistance,xlab="Spearman(GNScore, Intergenic Distance)",
			ylab="Spearman(comparative pred., distance model)", col=colors,pch=pch,
			main="C.  Raw and Processed Correlations");
	lines(0:-1,0:1,col="black",lty=2);
    }
    if (plot=="sig") {
	CompareHistogramList(log10(opAllMI[,c("anovabbfGNMinus","anovabbfGNScore","anovabbfGNWithin",
					"anovacfCOG","anovabbfMI")]),
			labels=c("GNMinus","GNScore","GNWithin","COG function","Phyl. Profiles"),
			breaks=c(-8:0)*2,
			lwd=c(1,1,2,1,1),
			xlab="log10(p) from generalized ANOVA",
			main="D.  ANOVA for Comparative Model of Strands",legendX=-12);
    }
}
	}

ExpVsUnsupervisedPlots <- function() {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(mfrow=c(1,2));
	ExpVsUnsupervisedPlot(ecP2,ecOp$name);
	ExpVsUnsupervisedPlot(bsP2,bsOp$name);
}

ExpVsUnsupervisedPlot <- function(P2,name,showUnknown=FALSE,title=TRUE) {
	d <- list(expAndUs=select(P2$orfPairs,P2$known==1 & P2$bOp),
				expNotUs=select(P2$orfPairs,P2$known==1 & !P2$bOp),
				nopNotUs=select(P2$orfPairs,P2$known==-1 & !P2$bOp),
				nopUs=select(P2$orfPairs,P2$known==-1 & P2$bOp));

	if(showUnknown) {
		d$unchUs <- select(P2$orfPairs,P2$known==0 & P2$bOp);
		d$unchNotUs <- select(P2$orfPairs,P2$known==0 & !P2$bOp);
	}
	counts <- sapply(d,length);
	names <- c("Known & Predicted", "Known Not & Predicted Not",
			"Known & Predicted Not", "Known Not & Predicted",
			"Unknown & Predicted", "Unknown & Predicted Not");
	names <- c("    Correct", "    Incorrect","    Correct", "    Incorrect");
	labels <- names;
	for (i in 1:length(labels)) { labels[i] <- paste(names[i], " (",counts[i],")", sep=""); }
	CompareHistogramList(d, xlab=paste("Microarray similarity (r) in ",name,sep=""),
				ylab="Proportion",
				ylim=c(0,.8),
				legend=FALSE,
				main=if(title) "Untrained Predictions vs. Known TUs" else "",
				breaks=c(-4:4)/4);
	legend(-1.1, .85, c("Known operons",labels[1],labels[2],
			"Known non-operons",labels[3],labels[4]),
			col=c(0,1,2,0,3,4),pch=c(-1,1,2,-1,3,4),lty=c(0,1,2,0,3,4),
			xjust=0,yjust=1);
}

ecTableComp <- function() { TableOpAOC(ecOp,ecAOC,ecP2); }
bsTableComp <- function() { TableOpAOC(bsOp,bsAOC,bsP2); }

ecTable <- function() { TableOp(ecOp,ecP2,header=FALSE); }
bsTable <- function() { TableOp(bsOp,bsP2,header=FALSE); }
hpTable <- function() { TableOp(hpOp,hpP2,header=FALSE); }
ctTable <- function() { TableOp(ctOp,ctP2,header=FALSE); }
scTable <- function() { TableOp(scOp,scP2,header=FALSE); }
hbTable <- function() { TableOp(hbOp,hbP2,header=FALSE); }

TableOp <- function(op,P2,skipEc=FALSE,pOp2=TRUE,header=TRUE) {
	cat("%Table for ",op$name,"\n",sep="");
	measures <- data.frame(col=c("pOp", "pOpLogistic", "GNMinus",
					"pOpDistance", "pOpDistanceEc", "Sep",
					"pOp1", "pOpEc", "pOp2" ),
				name=c("Untrained",
					"\\hspace{15pt} Logistic regression",
					"\\hspace{30pt} GNMinus",
					"\\hspace{15pt} Distance-only",
					"\\hspace{30pt} E.coli $p_d$",
					"\\hspace{30pt} Raw Sep.",
					"\\hspace{15pt} without CAI",
					"\\hspace{30pt} E.coli $p_d$",
					"\\hspace{15pt} relaxed assumption"
					),
				sign=1,skip=FALSE);
	measures$sign[select(1:length(measures$sign),measures$col=="Sep")] <- -1;
	if (!pOp2) {
		measures$skip <- measures$skip | measures$col == "pOp2";
	}
	P2 <- addEc(P2);
	op$preds <- addEc(op$preds);
	#if (!is.null(aoc)) {
	#	aoc$predsS <- addEc(aoc$predsS);
	#	aoc$predsD <- addEc(aoc$predsD);
	#}
	#if (op$genome==83333 && skipEc) {
	#	measures$skip[select(1:length(measures$skip),measures$col=="pOpEc")] <- TRUE;
	#	measures$skip[select(1:length(measures$skip),measures$col=="pOpDistanceEc")] <- TRUE;
	#}

	columns <- c("Measure (m)","$r_S(r_\\mu,m)$", "$r_S(r_\\mu,m>.5)$", "$r_S(p_{Op},m)$");

	f <- function(x) { sapply(x, function(y) { return(format(y,digits=3,nsmall=3)) }); }
	#rankThresh <- length(op$preds$pOp)*(1-op$fOperon); # fOperon above, 1-fOperon below
	measures <- select(measures,measures$skip==FALSE);
	for (m in 1:length(measures$sign)) {
		sign <- measures$sign[m];
		name <- levels(measures$name)[measures$name[m]];
		col <- levels(measures$col)[measures$col[m]];
		orfPairs <- P2$orfPairs;
		values <- P2[[col]];
		pOp <- P2$pOp;
		skip5 <- (col=="GNMinus" || col=="Sep");

		#if (col=="known") {
		#	orfPairs <- select(orfPairs, values!=0);
		#	pOp <- select(pOp, values !=0);
		#	values <- select(values, values!=0);
		#	thresh <- 0;
		#} else if (col=="pTrained") {
		#	thresh <- sign * sort(sign*aoc$pTrained)[rankThresh];
		#} else {
		#	thresh <- sign * sort(sign*op$preds[[col]])[rankThresh];
		#}
		#if (col == "pOp" || col == "pOpDistance") { thresh <- .5; }
		if (!skip5) {
			nUp <- countif(values > .5);
			if (nUp < .1*length(orfPairs) || nUp > .9*length(orfPairs)) { skip5 <- TRUE; }
		}
		#if (skip5) {
		#	D <- ks.test(select(orfPairs,values>.5), select(orfPairs,values<=.5))$thresh;
		#} else { D <- "--"; }

		#formatThresh <- if(col == "Sep") thresh else f(thresh);
		#if (col=="known") { formatThresh <- "--"; }
		r <- cor(rank(sign*values), rank(orfPairs));
		r5 <- if(skip5) "--" else cor(ifelse(values>.5,1,0),rank(orfPairs));
		cat(name, f(r), f(r5), f(cor(rank(sign*values),rank(pOp))), sep=" & ");
		cat("\\\\ \\hline\n");
	}
}

TableOpAOC <- function(op,aoc,P2) {
	cat("%Table vs known for ",op$name,"\n",sep="");
	measures <- data.frame(col=c("pOp","pOpDistance","pOpM","pM", "pTrained"),
				name=c("Untrained (Sequence-only)", "\\hspace{15pt} Distance-only",
						"Untrained with microarrays",
						"\\hspace {15pt} Microarray-only",
						"Trained (Sequence-only)"));
	P2$isop <- ifelse(P2$bOp,1,0);
	model <- glm(isop~(rank(orfPairs)+rank(sumD)+rank(S4))^2,family=binomial(logit),data=P2);
	P2$pM <- logodds2p(predict(model));
	P2$pOpM <- combineP(P2$pOp,logodds2p(predict(model)-mean(predict(model))));

	leftColumns <- c("Measure", "AOC", "Max. Acc.", "Def. Sens.", "Def. Spec.");
	f <- function(x) { sapply(x, function(y) { return(format(y,digits=3,nsmall=3)) }); }

	for (i in 1:nrow(measures)) {
		col <- levels(measures$col)[measures$col[i]];
		if(col=="pTrained") { cat("\\hline\n"); }
		name <- levels(measures$name)[measures$name[i]];
		if (col == "pM" || col == "pOpM") {
			opM <- select(P2[[col]],P2$known==1);
			nopM <- select(P2[[col]],P2$known==-1);
		} else {
			opM <- aoc$predsS[[col]];
			nopM <- aoc$predsD[[col]];
		}
		area <- makeaoc(opM,nopM,bigger=TRUE);
		defSens <- countPerIf(opM > .5);
		defSpec <- countPerIf(nopM <= .5);
		maxAcc <- max(AOC(opM,nopM,bigger=TRUE)$acc);
		cat(name, f(area), f(maxAcc), f(defSens), f(defSpec), sep=" & ");
		cat(" \\\\ \\hline\n");
	}
	cat("\\hline\n");
}

POpDistanceVSfOperon <- function() {
    old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
    par(mfrow=c(2,3),mar=c(5,4,2,.5)+0.1);
    namesSep <- c(-3, 0, 25, 50, 100, 200);
    namedSep <- sapply(namesSep, function(x){paste("pOpDistanceSep",ifelse(x<0,"Minus",""),abs(x),sep="");});
    namesSep <- ifelse(namesSep>0,namesSep,namesSep-1);
    for (i in 1:length(namesSep)) {
        plot(opAll[[namedSep[i]]],opAllNaive[[namedSep[i]]],
            col=ifelse(opAll$fPlus>.6,2,1), pch=ifelse(opAll$fPlus>.6,2,1),
            xlab=paste("p(d=",namesSep[i],") -- strand-smart P(Op|Same)",sep=""),
            ylab=paste("p(d=",namesSep[i],") -- neutral P(Op|Same)",sep=""));
        eqline();
    }
}

LeadingVSLaggingPlots <- function(showCanonical=FALSE) {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(mfrow=c(1,2),omi=rep(0,4),mgp=c(2,1,0),mar=c(3,3,3,1));
	col <- ifelse(opAll$fPlus>.6,2,1);
	plot(opAll$fLeadingSameGN,opAll$fLaggingSameGN,col=col,pch=col,
		main="A.  Proportion Conserved",
		xlab="Leading strand", ylab="Lagging strand");
	eqline();
	plot(opAll$medianLeadingSep,opAll$medianLaggingSep,col=col,pch=col,
		main="B.  Median Distance",
		xlab="Leading strand", ylab="Lagging strand");
	eqline();
	if (showCanonical) {
		plot(opAll$fLeadingCanonical,opAll$fLaggingCanonical,col=col,pch=col,
			main="C.  %Overlapping by 1 or 4",
			xlab="Leading strand", ylab="Lagging strand");
		eqline();
	}
}

SelectedPDPlot <- function(opKeep=select(opAll, opAll$pOpLogisticVSSep < median(opAll$pOpLogisticVSSep)))
{
	namesSep <- c(-10, -3, 0, 25, 50, 100, 150, 200, 300);
	namedSep <- sapply(namesSep, function(x) { paste("pOpDistanceSep",ifelse(x<0,"Minus",""),abs(x),sep=""); });
	seps <- opKeep[,namedSep];
	names(seps) <- namesSep;
	groupsSep <- combineColumns(seps);
	plot(groupsSep$type,groupsSep$x, xlab=paste("Intergenic Distance (d)"),ylab="Distance model p(d)" );
	n <- length(namesSep)-1;
	for (i in 1:n) { linePairs(i,seps[,i],i+1,seps[,i+1], col="grey"); }
}

GNScoreVSKnownVSUarray <- function(P2,name,...) {
	d <- list(Known=select(P2$orfPairs,P2$known==1),
			KnownNotN0=select(P2$orfPairs,P2$known==-1&P2$GNScore>0),
			KnownNot0=select(P2$orfPairs,P2$known==-1&P2$GNScore==0));
	n <- sapply(d,length);
	CompareHistogramList(d, breaks=c(-5:10)/10,
				labels=c(paste("Exp. known pairs (",n[1],")",sep=""),
					paste("Exp. not-operon pairs, conserved (",n[2],")",sep=""),
					paste("Exp. not-operon pairs, non-conserved (",n[3],")",sep="")),
				xlab=paste("Microarray correlation (",name,")",sep=""),
				legendX=-.5,ylim=c(0,.5),...);
}

AssumptionPlot <- function(cex=2/3) {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(mfrow=c(1,3),mar=c(5,2,3,2));
	CompareHistogramList(list(ecKnownNot=ecAOC$predsD$pSame,
					ecOpposing=select(ecOp$gcSC$pSame,ecOp$gcSC$Code != "Same"),
				bsKnownNot=bsAOC$predsD$pSame,
				bsOpposing=select(bsOp$gcSC$pSame,bsOp$gcSC$Code != "Same")),
				breaks=(0:20)/20,
				labels=c("Exp. non-operon (E.coli)", "Opposing strand (E.coli)",
					"Exp. non-operon (B.subtilis)", "Opposing strand (B.subtilis)"),
				xlab="P(Same | X) from Logistic Regression", col=c(1,2,5,4), lty=c(1,2,5,4),
				legendX=0,ylab="",cex=cex,
				main="A.  P(Same | X) vs. Exp. Pairs");
	GNScoreVSKnownVSUarray(ecP2,ecOp$name,cex=cex,ylab="",
		main="B.  Exp. vs. Microarrays (E.coli)");
	GNScoreVSKnownVSUarray(bsP2,bsOp$name,cex=cex,ylab="",
		main="C.  Exp. vs. Microarrays (B.subtilis)");
}

fOperonRow <- function(op,P2) {
	preds <- addEc(op$preds);
	P2 <- addEc(P2);
	ctest <- cortest2(P2$pOpDistance,P2$pOpDistanceEc,P2$orfPairs);
	ctestT <- cortest2(ifelse(P2$pOpDistance>=.5,1,0),ifelse(P2$pOpDistanceEc>=.5,1,0),
				P2$orfPairs);
	return(data.frame(fOperon=op$fOperon, meanpd = mean(preds$pOpDistance),
				meanpdEc=mean(preds$pOpDistanceEc),
				r1=ctest$r1,r2=ctest$r2,ppart=ctest$ppart,
				r1t=ctestT$r1,r2=ctestT$r2,ppart=ctestT$ppart,
				row.names=op$name));
}

# just an interesting factoid
fOperonTable <- function(ops=list(ecOp,bsOp,hpOp,ctOp,hbOp)) {
	return(rbind(fOperonRow(ecOp,ecP2),
			fOperonRow(bsOp,bsP2),
			fOperonRow(hpOp,hpP2),
			fOperonRow(ctOp,ctP2),
			fOperonRow(hbOp,hbP2)));
}

SingleUarrayPlot <- function(both=FALSE) {
	if(both) {
		old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
		par(mfrow=c(1,2));
	}
	if (!both) par(cex=5/3);

	plot(bsSop$medianOp,bsSop$medianNop,xlim=c(0,1.5),ylim=c(0,1.5),col="grey",
			xlab="Predicted operon pairs",
			ylab="Predicted not-operon pairs",
			main=if(both) "A. Median Difference in Log-ratio" else "Median Difference in Log-ratio");
	points(ecSop$medianOp,ecSop$medianNop,col="yellow",pch=2);
	points(hpSop$medianOp,hpSop$medianNop,col="blue",pch=3);
	points(ctSop$medianOp,ctSop$medianNop,col="red",pch=4);
	points(scSop$medianOp,scSop$medianNop,col="green",pch=5);
	eqline();
	legend(1.5,0.0,c("E.coli","B.subtilis","H.pylori","C.trachomatis","Synechocystis"),xjust=1,yjust=0,
			col=c("yellow","grey","blue","red","green"),pch=c(2,1,3,4,5), cex=1 );
	if (both) {
		plot(bsSop$rows,bsSop$allR,xlim=c(0,2500),ylim=c(-.5,.2),col="grey",
				xlab="# Pairs Considered", ylab="Spearman(p(Op), Difference)",
				main="B. Correlation with Predictions");
		points(ecSop$rows,ecSop$allR,col="yellow",pch=2);
		points(hpSop$rows,hpSop$allR,col="blue",pch=3);
		points(ctSop$rows,ctSop$allR,col="red",pch=4);
		points(scSop$rows,scSop$allR,col="green",pch=5);
		legend(2650,.225,c("E.coli","B.subtilis","H.pylori","C.trachomatis","Synechocystis"),
				col=c("yellow","grey","blue","red","green"),pch=c(2,1,3,4,5),
				xjust=1,yjust=1,cex=2/3);
	}
}

factoids <- function(save=FALSE) {
	if(save) {
		on.exit(sink(NULL));
		sink("workbook/UnsupervisedOperons/factoids");
	}
	f <- function(x) { format(x,digits=3,nsmall=3); }
	cat("Accuracy of split on high vs. low pSame\n");
	cat("E.coli: sens=", f(countPerIf(ecAOC$predsS$pSame > ecOp$threshPSame)),
		"spec=", f(countPerIf(ecAOC$predsD$pSame <= ecOp$threshPSame)),"\n");
	cat("B.subtilis: sens=", f(countPerIf(bsAOC$predsS$pSame > bsOp$threshPSame)),
		"spec=", f(countPerIf(bsAOC$predsD$pSame <= bsOp$threshPSame)),"\n");

	cat("\nB.subtilis, leading vs. lagging\n");
	ksBsSep <- ks.test(select(bsOp$preds$Sep,bsOp$preds$forward),select(bsOp$preds$Sep,!bsOp$preds$forward));
	cat("Sep: D-statistic",f(ksBsSep$statistic),"p",f(ksBsSep$p.value),"\n");
	cat("Percent with GNScore>0: leading",
			f(countPerIf(bsOp$preds$GNScore>0,bsOp$preds$forward)),
			"lagging",f(countPerIf(bsOp$preds$GNScore>0,bsOp$preds$forward)),
			"p",f(fisher.test(table(bsOp$preds$GNScore>0,bsOp$preds$forward))$p.value),
			"\n");

	cat("\nP(Op|Same) for outliers of our method vs. Moreno-Hagelsieb and Collado-Vides\n");
	mdiff <- abs(opAll$fOperon - .5 * opAll$fMoreno/ecOp$fMoreno);
	
	for(genome in select(opAll$genome[order(-mdiff)],-sort(-mdiff)>.2)) {
		row <- select(opAll,opAll$genome==genome);
		cat("Genome",genome,
			"\tMoreno",f(.5*row$fMoreno/ecOp$fMoreno),
			"fOp",f(row$fOperon),
			"fOpNaive",f(row$fOperonNaive),
			"P(Same)",f(row$fSame),
			"Leading",f(row$fPlus),
			"Name", levels(opAll$name)[row$name],
			"\n");
	}

	cat("\nB.subtilis, comparison of P(Op|Same) values\n");
	bsT <- CompareAOC(select(bsOp$preds$pOp,bsAOC$known==1),select(bsOp$preds$pOp,bsAOC$known==-1),
			select(bsOpNaive$preds$pOp,bsAOC$known==1),select(bsOpNaive$preds$pOp,bsAOC$known==-1));
	cat("P(Op|Same) : Strand-wise",f(bsOp$fOperon),"aoc",f(bsT$area1),
			"Strand-naive",f(bsOp$fOperonNaive),"aoc",f(bsT$area2),
			"p",f(bsT$p),"\n");
	bsT <- cortest2(bsP2$pOp,select(bsOpNaive$preds$pOp,!is.na(select(bsCors$orfPairs,bsOp$gcSC$same))),
			bsP2$orfPairs,spearman=TRUE);
	cat("Strand-wise vs. naive on microarrays: strand-wise",bsT$r1,"-naive",bsT$r2,"p",f(bsT$ppart),
		"from r(diff)",f(bsT$rpart),"\n");

	cat("\nSupervised vs. Untrained predictions: accuracy on training data & microarrays (all pairs):\n");
	ecT <- cortest2(ecP2$pTrained,ecP2$pOp,ecP2$orfPairs,spearman=TRUE);
	bsT <- cortest2(bsP2$pTrained,bsP2$pOp,bsP2$orfPairs,spearman=TRUE);
	cat("E.coli: Spearman(pTrained,r)",f(ecT$r1),"Spearman(pOp,r)",f(ecT$r2),
		"p",f(ecT$ppart),"from r(diff)",f(ecT$rpart),"\n");
	cat("B.subt: Spearman(pTrained,r)",f(bsT$r1),"Spearman(pOp,r)",f(bsT$r2),
		"p",f(bsT$ppart),"from r(diff)",f(bsT$rpart),"\n");
	ecT <- CompareAOC(ecAOC$predsS$pTrained,ecAOC$predsD$pTrained,ecAOC$predsS$pOp,ecAOC$predsD$pOp);
	bsT <- CompareAOC(bsAOC$predsS$pTrained,bsAOC$predsD$pTrained,bsAOC$predsS$pOp,bsAOC$predsD$pOp);
	cat("E.coli: trained aoc",f(ecT$area1),"unsup aoc",f(ecT$area2),"p",f(ecT$p),
			"max acc trained", f(max(ecAOC$aocTrained$acc)),
			"max acc unsup", f(max(ecAOC$aocUntrained$acc)), "\n");
	cat("B.subt: trained aoc",f(bsT$area1),"unsup aoc",f(bsT$area2),"p",f(bsT$p),
			"max acc trained", f(max(bsAOC$aocTrained$acc)),
			"max acc unsup", f(max(bsAOC$aocUntrained$acc)), "\n");


	cat("\nSup distance model vs. E.coli distance model (using microarrays)\n");
	d <- list(ecol=ecP2,bsub=bsP2,hpyl=hpP2,ctra=ctP2,syne=scP2,halo=hbP2);
	for (name in names(d)) {
		P2 <- addEc(d[[name]]);
		test <- cortest2(P2$pOpDistance,P2$pOpDistanceEc,P2$orfPairs,spearman=TRUE);
		cat(name,"rUs", f(test$r1), "rEc", f(test$r2), "p", f(test$ppart),
			"from r(diff)", f(test$rpart), "\n");
	}

	cat("\nConservation (%GNScore>0) vs. Sep in H.pylori and Halobacterium\n");
	cat("Halobacterium 25-50",
		f(countPerIf(hbOp$preds$GNScore>0,hbOp$preds$Sep>=25&hbOp$preds$Sep<=50)),
		"(",countif(hbOp$preds$GNScore>0 & hbOp$preds$Sep>=25 & hbOp$preds$Sep<=50),
		"/",countif(hbOp$preds$Sep>=25 & hbOp$preds$Sep<=50),")",
		"other",
		f(countPerIf(hbOp$preds$GNScore>0,!(hbOp$preds$Sep>=25&hbOp$preds$Sep<=50))),
		"(",countif(hbOp$preds$GNScore>0 & !(hbOp$preds$Sep>=25 & hbOp$preds$Sep<=50)),")",
		"/",countif(!(hbOp$preds$Sep>=25 & hbOp$preds$Sep<=50)),
		"p",
		f(fisher.test(table(hbOp$preds$GNScore>0,hbOp$preds$Sep>=25&hbOp$preds$Sep<=50))$p.value),
		"\n\tvs. E.coli 25-50",
		f(countPerIf(ecOp$preds$GNScore>0,ecOp$preds$Sep>=25&ecOp$preds$Sep<=50)),
		"(",countif(ecOp$preds$GNScore>0 & ecOp$preds$Sep>=25 & ecOp$preds$Sep<=50),
		"/",countif(ecOp$preds$Sep>=25 & ecOp$preds$Sep<=50),")",
		"other",
		f(countPerIf(ecOp$preds$GNScore>0,!(ecOp$preds$Sep>=25&ecOp$preds$Sep<=50))),
		"(",countif(ecOp$preds$GNScore>0 & !(ecOp$preds$Sep>=25 & ecOp$preds$Sep<=50)),
		"/",countif(!(ecOp$preds$Sep>=25 & ecOp$preds$Sep<=50)),")",
		"p",
		f(fisher.test(table(ecOp$preds$GNScore>0,ecOp$preds$Sep>=25&ecOp$preds$Sep<=50))$p.value),"\n");
	cat("H.pylori 50-100",
		f(countPerIf(hpOp$preds$GNScore>0,hpOp$preds$Sep>=50&hpOp$preds$Sep<=100)),
		"(",countif(hpOp$preds$GNScore>0 & hpOp$preds$Sep>=50 & hpOp$preds$Sep<=100),
		"/",countif(hpOp$preds$Sep>=50 & hpOp$preds$Sep<=100),")",
		"other",
		f(countPerIf(hpOp$preds$GNScore>0,!(hpOp$preds$Sep>=50&hpOp$preds$Sep<=100))),
		"(",countif(hpOp$preds$GNScore>0 & !(hpOp$preds$Sep>=50 & hpOp$preds$Sep<=100)),")",
		"/",countif(!(hpOp$preds$Sep>=50 & hpOp$preds$Sep<=100)),
		"p",
		f(fisher.test(table(hpOp$preds$GNScore>0,hpOp$preds$Sep>=50&hpOp$preds$Sep<=100))$p.value),
		"odds ratios",
		f(fisher.test(table(hpOp$preds$GNScore>0,hpOp$preds$Sep>=50&hpOp$preds$Sep<=100))$conf.int),
		"\n\tvs. E.coli 50-100",
		f(countPerIf(ecOp$preds$GNScore>0,ecOp$preds$Sep>=50&ecOp$preds$Sep<=100)),
		"(",countif(ecOp$preds$GNScore>0 & ecOp$preds$Sep>=50 & ecOp$preds$Sep<=100),
		"/",countif(ecOp$preds$Sep>=50 & ecOp$preds$Sep<=100),")",
		"other",
		f(countPerIf(ecOp$preds$GNScore>0,!(ecOp$preds$Sep>=50&ecOp$preds$Sep<=100))),
		"(",countif(ecOp$preds$GNScore>0 & !(ecOp$preds$Sep>=50 & ecOp$preds$Sep<=100)),
		"/",countif(!(ecOp$preds$Sep>=50 & ecOp$preds$Sep<=100)),")",
		"p",
		f(fisher.test(table(ecOp$preds$GNScore>0,ecOp$preds$Sep>=50&ecOp$preds$Sep<=100))$p.value),
		"odds ratios",
		f(fisher.test(table(ecOp$preds$GNScore>0,ecOp$preds$Sep>=50&ecOp$preds$Sep<=100))$conf.int),
		"\n");

	cat("\nHp and Sc have operons:\n")
	cat("P(Op|Same) for H.pylori",f(hpOp$fOperon),"C.jejuni",f(select(opAll$fOperon,opAll$genome==197)),"\n");
	hpVScj <- select(gncAll,gncAll$TaxId1==hpOp$genome & gncAll$TaxId2==197);
	cat("H.pylori vs. C.jejuni same adj",f(hpVScj$nSameA/hpOp$nSame),hpVScj$nSameA,"/",hpOp$nSame,
		"control adj",f((hpVScj$nDivA+hpVScj$nConvA)/(hpOp$nDiv+hpOp$nConv)),
		hpVScj$nDivA+hpVScj$nConvA, "/", hpOp$nDiv+hpOp$nConv,
		"p",
		f(prop.test(c(hpVScj$nSameA,hpVScj$nDivA+hpVScj$nConvA),c(hpOp$nSame,hpOp$nDiv+hpOp$nConv))$p.value),
		"fHpInCj", f((2*hpVScj$nBoth + hpVScj$nOne)/(2*hpVScj$nTot1)), "\n");
	cat("P(Op|Same) for Synechocystis",f(scOp$fOperon),
		"Nostoc sp. PCC7120",f(select(opAll$fOperon,opAll$genome==103690)),"\n");
	scVSnc <- select(gncAll,gncAll$TaxId1==scOp$genome & gncAll$TaxId2==103690);
	cat("Synechocystis vs. Nostoc same adj",f(scVSnc$nSameA/scOp$nSame),scVSnc$nSameA,"/",scOp$nSame,
		"control adj",f((scVSnc$nDivA+scVSnc$nConvA)/(scOp$nDiv+scOp$nConv)),
		scVSnc$nDivA+scVSnc$nConvA, "/", scOp$nDiv+scOp$nConv,
		"p",
		f(prop.test(c(scVSnc$nSameA,scVSnc$nDivA+scVSnc$nConvA),c(scOp$nSame,scOp$nDiv+scOp$nConv))$p.value),
		"fScInNc", f((2*scVSnc$nBoth + scVSnc$nOne)/(2*scVSnc$nTot1)), "\n");
	scVSsc <- select(gncAll,gncAll$TaxId1==scOp$genome & gncAll$TaxId2==84588);
	# NCBIs taxonomy implies Synechococcus is closer than Nostoc, but that isn't apparent from gene content or gene neighbors
	cat("Synechocystis vs. Synechococcus WH 8102 same adj",f(scVSsc$nSameA/scOp$nSame),scVSsc$nSameA,"/",scOp$nSame,
		"control adj",f((scVSsc$nDivA+scVSsc$nConvA)/(scOp$nDiv+scOp$nConv)),
		scVSsc$nDivA+scVSsc$nConvA, "/", scOp$nDiv+scOp$nConv,
		"p",
		f(prop.test(c(scVSsc$nSameA,scVSsc$nDivA+scVSsc$nConvA),c(scOp$nSame,scOp$nDiv+scOp$nConv))$p.value),
		"fScInNc", f((2*scVSsc$nBoth + scVSsc$nOne)/(2*scVSsc$nTot1)), "\n");

	cat("\nLogistic regression effectiveness Spearman: ",f(range(opAll$rLogistic)),"\n");

	cat("\n");
}

genomeTable <- function() {
	cat("\\begin{tabular}\n");
	cat("{l|r|c|c|c|c|r}\n");
	cat("Name","TaxId","Source","Cluster","$P(Same)$","$P(Op|Same)$","$n_{Same}$",sep=" & ");
	cat("\\\\ \\hline\n");

	cluster <- read.delim("operons/out/clusterConvAdj5");
	f <- function(x) { sapply(x, function(y) { return(sprintf("%.3f",y)) }); }
	nshown <- 0;
	for (i in order(opAll$name)) {
		row <- opAll[i,];
		if (!is.null(row)) {
			nshown <- nshown+1;
			if(nshown%%68 ==0) {
				cat("\\end{tabular}\n");
				cat("\\begin{tabular}\n");
				cat("{l|r|c|c|c|c|r}\n");
				cat("Name","TaxId","Source","Cluster",
					"$P(Same)$","$P(Op|Same)$","$n_{Same}$",sep=" & ");
				cat("\\\\ \\hline\n");
			}
			clusterId <- select(cluster$Cluster,cluster$TaxId==row$genome);
			names2 <- select(levels(cluster$Genome)[cluster$Genome],cluster$Cluster==clusterId);
			clusterSize <- select(cluster$Size,cluster$TaxId==row$genome);
			clusterDesc <- paste(substr(names2[1],1,5),clusterSize, sep="");
			if(clusterSize==1) clusterDesc <- "--";
			cat(paste(row$nameShort), row$genome, paste(row$source),
				clusterDesc, f(row$fSame),f(row$fOperon),row$nSame,sep=" & ");
			cat("\\\\ \n");
		}
	}
	cat("\\end{tabular}\n");
}

uarraySubTable <- function(source,op,Sop,SEs) {
	removeslash <- function(x) {
		r <- regexpr("/[^/.]+\\.",x);
		return(substr(x,r+1,r+attr(r,"match.length")-2));
	}
	f <- function(x) { sapply(x, function(y) { return(if(!is.numeric(y)) "--" else sprintf("%.3f",y)); } ) }
	cat("\\begin{tabular}\n");
	cat("{l|l|r|c|l}\n");
	cat("\\multicolumn{5}{l}{{\\bf ",op$name,"}}\\\\ \n",sep="");
	cat("Source","Condition","$n_{pairs}$","$Spearman(p,|\\delta|)$","Publication",sep=" & ");
	cat("\\\\ \\hline\n");
	for (i in 1:nrow(Sop)) {
		index <- Sop$dim[i]-1;
		if (levels(Sop$cond)[Sop$cond[i]] != levels(SEs$condition)[SEs$condition[index]]) {
			warning("different conditions: ",levels(Sop$cond)[Sop$cond[i]],
				 levels(SEs$condition)[SEs$condition[index]]);
		}
		cite <- levels(SEs$citation)[SEs$citation[index]];
		cond <- levels(Sop$cond)[Sop$cond[i]];
		if (source=="SMD" || source=="KEGG") {
			source2 <- paste(source,":",removeslash(levels(SEs$file)[SEs$file[index]]),sep="");
		} else {
			source2 <- source;
			if (cite != "--") { if(substr(cite,1,4)=="Nich" && cond != "Growth") { cite <- "--"; } }
		}
		if (i%%50==0) {
			cat("\\end{tabular}\n");
			cat("\\begin{tabular}\n");
			cat("{l|l|r|c|l}\n");
			cat("Source","Condition","$n_{pairs}$","$Spearman(p,|\\delta|)$","Publication",sep=" & ");
			cat("\\\\ \\hline\n");
		}
		cat(source2, cond, if(is.numeric(Sop$rows[i])) Sop$rows[i] else "--", f(Sop$allR[i]), cite, sep=" & ");
		cat("\\\\ \\hline\n");
	}	
	cat("\\end{tabular}\n");
}

uarrayTable <- function() {
	ecSEs <- read.delim("operons/data/ecSEs");
	bsSEs <- read.delim("operons/data/bsSEs");
	hpSEs <- read.delim("operons/data/hpSEs");
	ctSEs <- read.delim("operons/data/ctSEs");
	scSEs <- read.delim("operons/data/scSEs");

	uarraySubTable("SMD",ecOp,ecSop,ecSEs);
	uarraySubTable("SMD",bsOp,bsSop,bsSEs);
	uarraySubTable("SMD",hpOp,hpSop,hpSEs);
	uarraySubTable("KEGG",scOp,scSop,scSEs);
	uarraySubTable("TN \\& RS",ctOp,ctSop,ctSEs);
	hbSop <- data.frame(cond=c("UV resistance (5 experiments)",
				"Light/dark response (3 experiments)",
				"bop/bat genetic perturbations (3 experiments)",
				"day/night entrainment (33 experiments)"),
			dim=(2:5), citation=c("--","--","Baliga NS et al. (2002) PNAS 99:14913-8","--"),
			allR=rep(NA,4),rows=rep(NA,4));
	uarraySubTable("RN \\& NB",hbOp,hbSop,data.frame(condition=hbSop$cond,citation=hbSop$citation));
}

fOperonVsUarrayPlots <- function(n=4,useDensity=TRUE,bigMar=FALSE) {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(mfrow=c(2,3));
	if(bigMar) par(omi=rep(.5,4),mar=rep(.1,4));
	F <- fOperonVsUarrayPlot(ecOp,ecP2,ecCors,n=n,useDensity=useDensity);
	F <- fOperonVsUarrayPlot(bsOp,bsP2,bsCors,n=n,useDensity=useDensity,legend=TRUE);
	F <- fOperonVsUarrayPlot(hpOp,hpP2,hpCors,n=n,useDensity=useDensity);
	F <- fOperonVsUarrayPlot(ctOp,ctP2,ctCors,n=n,useDensity=useDensity);
	F <- fOperonVsUarrayPlot(scOp,scP2,scCors,n=n,useDensity=useDensity);
	F <- fOperonVsUarrayPlot(hbOp,hbP2,hbCors,n=1,useDensity=useDensity);
}

fOperonVsUarrayPlot <- function(op,P2,cors,n=4,legend=FALSE,useDensity=TRUE) {
	breaks <- seq(-1,1,.1);
	F <- fOperonVsUarray(op,cors,n=n,useDensity=useDensity);
	# P(X|S) = f*P(X|H) + (1-f)*P(X|C), so
	# P(X|S)-P(X|C) = f*(P(X|H)-P(X|C))
	model <- summary(lm(I(same-controlR)~I(highR-controlR)+0,F));
	coeff <- model$coefficients[1,1];
	err <- model$coefficients[1,2];
	mult <- nrow(P2)*length(F$x)/(length(breaks)-1);
	hist(P2$orfPairs,breaks,xlab="Expression Similarity (Pearson r)",ylab="Counts",ylim=c(0,max(c(F$highR,F$controlR))*mult),
			main=paste(op$name,"\n\nBest Fit P(Op|Same) = ",format(coeff,digits=2,nsmall=2),
				"\n\nSequence-based estimate = ",format(op$fOperon,digits=2,nsmall=2),
				"\n\nE.coli-based estimate = ",format(op$fMoreno*.5/ecOp$fMoreno,digits=2,nsmall=2),sep=""),
			cex.main=3/4);
	lines(F$x,F$highR*mult,col=3);
	lines(F$x,F$controlR*mult,col=2,lty=2);
	lines(F$x,F$mix*mult,col="grey",lty=2);
	lines(F$x,F$same*mult);
	lines(F$x,(F$highR*coeff + F$controlR*(1-coeff))*mult,col="grey",lwd=2);
	if(legend) {
		legend(-1.1,max(F$highR)*mult,c("Same-strand pairs","Confident predictions","Opposing-strand pairs",
						"Best fit","Seq-based prediction"),
			col=c("black","green","red","grey","grey"),lwd=c(1,1,1,2,1),lty=c(1,1,2,1,2));
	}
	cat("fOperon estimate for",op$name,"=",coeff,"vs",op$fOperon,
			"total err of fit",sum(abs(F$highR*coeff+F$controlR*(1-coeff)-F$same)),"\n");
}

CompareAllToEc <- function(save=FALSE) {
	out <- rbind(CompareToEc(ecOp,ecP2),
			CompareToEc(bsOp,bsP2),
			CompareToEc(hpOp,hpP2),
			CompareToEc(ctOp,ctP2),
			CompareToEc(scOp,scP2),
			CompareToEc(hbOp,hbP2));
	if(save) { writeDelim(out,"workbook/UnsupervisedOperons/CompareAlltoEc"); }
	return(out);
}

CompareToEc <- function(op,P2) {
	P2 <- addEc(P2);
	tAll <- cortest2(ifelse(P2$pOpDistance>.5,1,0),ifelse(P2$pOpDistanceEc>.5,1,0),P2$orfPairs,spearman=TRUE);
	if(is.null(P2$sumD1) || min(P2$sumD1)==max(P2$sumD1)) {
		tSAbove <- tAbove <- list(r1=NA,r2=NA,r12=NA,rpart=NA,ppart=NA);
	} else {
		P2data <- select(P2,P2$sumD1>median(P2$sumD1));
		tAbove <- cortest2(ifelse(P2data$pOpDistance>.5,1,0),ifelse(P2data$pOpDistanceEc>.5,1,0),
				P2data$orfPairs,spearman=TRUE);
		tSAbove <- cortest2(P2data$pOpDistance,P2data$pOpDistanceEc,P2data$orfPairs,spearman=TRUE);
	}
	tS <- cortest2(P2$pOpDistance,P2$pOpDistanceEc,P2$orfPairs,spearman=TRUE);

	preds <- addEc(op$preds);
	return(data.frame(name=op$name,fOp=op$fOperon,meanOp=mean(preds$pOpDistance),meanOpEc=mean(preds$pOpDistanceEc),
				perOp=countPerIf(preds$pOpDistance>.5),perOpEc=countPerIf(preds$pOpDistanceEc>.5),
				allr12=tAll$r12,allrpart=tAll$rpart,allp=tAll$ppart,
				abr12=tAbove$r12,abrpart=tAbove$rpart,abp=tAbove$ppart,
				sr12=tS$r12,srpart=tS$rpart,sp=tS$ppart,
				sabr12=tSAbove$r12,sabrpart=tSAbove$rpart,sabp=tSAbove$ppart));
}

JackknifePlot <- function() {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(cex=.5);
	plot(c(1,23),c(0,1),col=0,axes=FALSE,ylab="Estimates from microarrays",xlab="");
	legend(1,0,c("Sequence-based expectation","Jackknife on experimental conditions","Jackknife 95% confidence",
			"Resampling genes, 95% confidence", "Accuracy on known operons"),
			col=c("red","black","green","black","orange"),
			pch=c(7,1,-1,-1,2), lty=c(0,0,1,1,0), lwd=c(0,0,2,1,0), yjust=0);
	at <- c(1,2,3,5,6,7,9,10,11,13,14,15,17,18,19,21,22,23);
	axis(1,at=at, las=2,
		labels=rep(c("prior","accT","accF"),6));
	axis(1,at=2+4*(0:5),labels=c(ecOp$name,bsOp$name,"H.pylori","C.trachomatis","Synechocystis","Halobacterium"),
				line=2,col=0);
	axis(2);
	JackknifePoints(ecOp,ecP2,ecCors,ecAcc,offset=0); ResampleArrows(ecAcc2,offset=0);
	JackknifePoints(bsOp,bsP2,bsCors,bsAcc,offset=4); ResampleArrows(bsAcc2,offset=4);
	JackknifePoints(hpOp,hpP2,hpCors,hpAcc,offset=8); ResampleArrows(hpAcc2,offset=8);
	JackknifePoints(ctOp,ctP2,ctCors,ctAcc,offset=12); ResampleArrows(ctAcc2,offset=12);
	JackknifePoints(scOp,scP2,scCors,scAcc,offset=16); ResampleArrows(scAcc2,offset=16);
	JackknifePoints(hbOp,hbP2,hbCors,NULL,offset=20,n=1); ResampleArrows(hbAcc2,offset=20);
	points(c(2,3,6,7),c(countPerIf(ecAOC$predsS$bOp),1-countPerIf(ecAOC$predsD$bOp),
					countPerIf(bsAOC$predsS$bOp),1-countPerIf(bsAOC$predsD$bOp)),
			col="orange",pch=2);
}

ResampleArrows <- function(acc2,offset=0) {
	f <- function(x,y) { arrows(x,y[1],x,y[2],code=3,angle=90,length=1/18); }
	f(offset+1,t.test(acc2$fOp)$conf.int);
	f(offset+2,t.test(acc2$accT)$conf.int);
	f(offset+3,t.test(acc2$accF)$conf.int);
}

JackknifePoints <- function(op,P2,cors,acc,n=NULL,useDensity=NULL, offset=0) {
	F <- fOperonVsUarray(op,cors,n=n,useDensity=useDensity);

	fOp <- lm(I(same-controlR)~I(highR-controlR)+0,F)$coefficients[1];
	accT <- lm(I(op-controlO)~I(highO-controlO)+0,F)$coefficients[1];
	accF <- lm(I(nop-highN)~I(controlN-highN)+0,F)$coefficients[1];

	if (!is.null(acc)) {
		# This approach is statistically, recommended, but produces
		# very odd results, so it is not shown
		m <- nrow(acc);
		esfOp <- m*fOp - (m-1)*acc$fOp;
		esaccT <- m*accT - (m-1)*acc$accT;
		esaccF <- m*accF - (m-1)*acc$accF;

		mult <- sqrt((m-1)*m/(m+1));
		f <- function(x,y,mult) {
			int <- mean(y) + mult*t.test(y-mean(y))$conf.int;
			arrows(x,int[1],x,int[2],angle=90,code=3,length=1/12,lwd=2,col="green");
		}
		f(offset+1,acc$fOp,mult);
		f(offset+2,acc$accT,mult);
		f(offset+3,acc$accF,mult);
		points(offset+jitter(c(rep(1,m),rep(2,m),rep(3,m)),factor=1/4), c(acc$fOp,acc$accT,acc$accF));
	} else {
		points(offset+c(1,2,3),c(fOp,accT,accF),col="darkgrey");
	}
	points(offset+1,op$fOperon,col=2,pch=7);
	points(offset+2,mean(P2$pOp[P2$bOp]),col=2,pch=7);
	points(offset+3,1-mean(P2$pOp[!P2$bOp]),col=2,pch=7);
}

CAIplots <- function() {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(mfrow=c(4,4));
	CAIplot(ecOp$gcSC,ecCors,main="E.coli K12");
	CAIplot(bsOp$gcSC,bsCors,main="B.subtilis");
	CAIplot(hpOp$gcSC,hpCors,main="H.pylori");
	CAIplot(scOp$gcSC,scCors,main="Synechocystis");
}

CAIplot <- function(gc,cors,main=main,...) {
	# only need first member to be present; assume values of 0 as likely to indicate
	# absence from chip or spot problems as true absence of expression
	C2 <- select(cbind(gc,cors,S1=cors$S1c+cors$S1t), cors$S1c>0 & cors$S1t>0 & !is.na(gc$CAI));
	model <- loess(rank(S1)~rank(CAI),C2)
	plot(C2$CAI,model$y,col="grey",
			main=paste(main,"level\nSpearman=",sprintf("%.3f",cor(rank(C2$S1),rank(C2$CAI)))),
			xlab="CAI",ylab=paste("ranked level (",max(C2$n)," exp.)"),...);
	sortedLines(C2$CAI,model$fitted,col="red");
	plot(cut4(C2$CAI),rank(C2$S1),xlab="CAI quartile",ylab="ranked level");

	modelD <- loess(rank(sumD1)~rank(CAI),C2);
	plot(C2$CAI,modelD$y,col="grey",
			main=paste(main,"variability\nSpearman=",sprintf("%.3f",cor(rank(C2$sumD1),rank(C2$CAI)))),
			xlab="CAI",ylab=paste("ranked variability (",max(C2$n)," exp.)"),...);
	sortedLines(C2$CAI,modelD$fitted,col="red");
	plot(cut4(C2$CAI),rank(C2$sumD1),xlab="CAI quartile",ylab="ranked variability");
}

SepVsUarrayPlots <- function() {
	old.par <- par(no.readonly=TRUE); on.exit(par(old.par));
	par(mfrow=c(2,3));
	SepVsUarrayPlot(ecP2,main="E.coli K12",legend=TRUE);
	SepVsUarrayPlot(bsP2,main="B.subtilis");
	SepVsUarrayPlot(hpP2,main="H.pylori");
	SepVsUarrayPlot(scP2,main="Synechocystis");
	SepVsUarrayPlot(ctP2,main="C.trachomatis");
	SepVsUarrayPlot(hbP2,main="Halobacterium");
}

SepVsUarrayPlot <- function(P2,main="",legend=FALSE) {
	LoessAndCut(P2$Sep,P2$orfPairs,ylim=c(-1,1),
			xlim=c(-100,400),xlab="Intergenic Distance", ylab="Pearson correlation (r)",main=main);
	sortedLines(P2$Sep,P2$pSameD,col="green",lty=2,lwd=2);
	if(legend) legend(-100,-1,c("Pearson correlation (r)", "   Smoothed fit", "   Boxplots", "Distance model"),
			col=c("grey","red","black","green"), lty=c(-1,1,1,2), pch=c(1,-1,1,-1),
				lwd=c(1,1,1,2), yjust=0);
}
